@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -141,24 +141,31 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	DBstart();
 
 	$newHostPrototype = [
-		'host' => getRequest('host'),
-		'name' => getRequest('name'),
+		'host' => getRequest('host', ''),
+		'name' => (getRequest('name', '') === '') ? getRequest('host', '') : getRequest('name', ''),
 		'status' => getRequest('status', HOST_STATUS_NOT_MONITORED),
 		'groupLinks' => [],
 		'groupPrototypes' => [],
-		'templates' => getRequest('templates', []),
-		'inventory' => [
-			'inventory_mode' => getRequest('inventory_mode')
-		]
+		'templates' => getRequest('templates', [])
 	];
 
+	// If 'inventory' is present, API requires 'inventory_mode' to have a value, but templated prototypes don't have it.
+	if (hasRequest('inventory_mode')) {
+		$newHostPrototype['inventory']['inventory_mode'] = getRequest('inventory_mode');
+	}
+
+	// API requires 'templateid' property.
+	if ($newHostPrototype['templates']) {
+		$newHostPrototype['templates'] = zbx_toObject($newHostPrototype['templates'], 'templateid');
+	}
+
 	// add custom group prototypes
 	foreach (getRequest('group_prototypes', []) as $groupPrototype) {
 		if (!$groupPrototype['group_prototypeid']) {
 			unset($groupPrototype['group_prototypeid']);
 		}
 
-		if (!zbx_empty($groupPrototype['name'])) {
+		if ($groupPrototype['name'] !== '') {
 			$newHostPrototype['groupPrototypes'][] = $groupPrototype;
 		}
 	}
@@ -252,6 +259,15 @@ elseif (hasRequest('action') && getRequest('action') == 'hostprototype.massdelet
 	show_messages($result, _('Host prototypes deleted'), _('Cannot delete host prototypes'));
 }
 
+if (hasRequest('action') && hasRequest('group_hostid') && !$result) {
+	$host_prototypes = API::HostPrototype()->get([
+		'output' => [],
+		'hostids' => getRequest('group_hostid'),
+		'editable' => true
+	]);
+	uncheckTableRows($discoveryRule['itemid'], zbx_objectValues($host_prototypes, 'hostid'));
+}
+
 $config = select_config();
 
 /*
@@ -273,7 +289,8 @@ if (hasRequest('form')) {
 			'groupPrototypes' => getRequest('group_prototypes', [])
 		],
 		'groups' => [],
-		'show_inherited_macros' => getRequest('show_inherited_macros', 0)
+		'show_inherited_macros' => getRequest('show_inherited_macros', 0),
+		'templates' => []
 	];
 
 	// add already linked and new templates
@@ -334,32 +351,12 @@ if (hasRequest('form')) {
 		}
 	}
 
-	$templateids = [];
-
-	// Add parent templates.
-	if ($data['host_prototype']['templateid'] != 0) {
-		$data['parents'] = [];
-		$hostPrototypeId = $data['host_prototype']['templateid'];
-		while ($hostPrototypeId) {
-			$parentHostPrototype = API::HostPrototype()->get([
-				'output' => ['itemid', 'templateid'],
-				'selectParentHost' => ['hostid', 'name'],
-				'selectDiscoveryRule' => ['itemid'],
-				'hostids' => $hostPrototypeId
-			]);
-			$parentHostPrototype = reset($parentHostPrototype);
-			$hostPrototypeId = null;
-
-			if ($parentHostPrototype) {
-				$data['parents'][] = $parentHostPrototype;
-				$hostPrototypeId = $parentHostPrototype['templateid'];
-				$templateids[] = $parentHostPrototype['parentHost']['hostid'];
-			}
-		}
-	}
+	$data['templates'] = makeHostPrototypeTemplatesHtml($data['host_prototype']['hostid'],
+		getHostPrototypeParentTemplates([$data['host_prototype']])
+	);
 
 	// Select writable templates
-	$templateids = array_merge(zbx_objectValues($data['host_prototype']['templates'], 'templateid'), $templateids);
+	$templateids = zbx_objectValues($data['host_prototype']['templates'], 'templateid');
 	$data['host_prototype']['writable_templates'] = [];
 
 	if ($templateids) {
@@ -410,7 +407,10 @@ else {
 		->setArgument('parent_discoveryid', $data['parent_discoveryid']);
 
 	$data['paging'] = getPagingLine($data['hostPrototypes'], $sortOrder, $url);
-	// fetch templates linked to the prototypes
+
+	$data['parent_templates'] = getHostPrototypeParentTemplates($data['hostPrototypes']);
+
+	// Fetch templates linked to the prototypes.
 	$templateids = [];
 	foreach ($data['hostPrototypes'] as $hostPrototype) {
 		$templateids = array_merge($templateids, zbx_objectValues($hostPrototype['templates'], 'templateid'));
@@ -424,32 +424,6 @@ else {
 	]);
 	$data['linkedTemplates'] = zbx_toHash($linkedTemplates, 'templateid');
 
-	// fetch source templates and LLD rules
-	$hostPrototypeSourceIds = getHostPrototypeSourceParentIds(zbx_objectValues($data['hostPrototypes'], 'hostid'));
-	if ($hostPrototypeSourceIds) {
-		$hostPrototypeSourceTemplates = DBfetchArrayAssoc(DBSelect(
-			'SELECT h.hostid,h2.name,h2.hostid AS parent_hostid'.
-			' FROM hosts h,host_discovery hd,items i,hosts h2'.
-			' WHERE h.hostid=hd.hostid'.
-				' AND hd.parent_itemid=i.itemid'.
-				' AND i.hostid=h2.hostid'.
-				' AND '.dbConditionInt('h.hostid', $hostPrototypeSourceIds)
-		), 'hostid');
-		foreach ($data['hostPrototypes'] as &$hostPrototype) {
-			if ($hostPrototype['templateid']) {
-				$sourceTemplate = $hostPrototypeSourceTemplates[$hostPrototypeSourceIds[$hostPrototype['hostid']]];
-				$hostPrototype['sourceTemplate'] = [
-					'hostid' => $sourceTemplate['parent_hostid'],
-					'name' => $sourceTemplate['name']
-				];
-				$sourceDiscoveryRuleId = get_realrule_by_itemid_and_hostid($discoveryRule['itemid'], $sourceTemplate['hostid']);
-				$hostPrototype['sourceDiscoveryRuleId'] = $sourceDiscoveryRuleId;
-				$templateids[] = $sourceTemplate['parent_hostid'];
-			}
-		}
-		unset($hostPrototype);
-	}
-
 	foreach ($data['linkedTemplates'] as $linked_template) {
 		foreach ($linked_template['parentTemplates'] as $parent_template) {
 			$templateids[] = $parent_template['templateid'];
