@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,9 +26,9 @@
 class CFrontendSetup {
 
 	const MIN_PHP_VERSION = '5.4.0';
-	const MIN_PHP_MEMORY_LIMIT = 134217728; // 128*1024*1024
-	const MIN_PHP_POST_MAX_SIZE = 16777216; // 16*1024*1024
-	const MIN_PHP_UPLOAD_MAX_FILESIZE = 2097152; // 2*1024*1024
+	const MIN_PHP_MEMORY_LIMIT = '134217728'; // 128 * ZBX_MEBIBYTE;
+	const MIN_PHP_POST_MAX_SIZE = '16777216'; // 16 * ZBX_MEBIBYTE;
+	const MIN_PHP_UPLOAD_MAX_FILESIZE = '2097152'; // 2 * ZBX_MEBIBYTE;
 	const MIN_PHP_MAX_EXECUTION_TIME = 300;
 	const MIN_PHP_MAX_INPUT_TIME = 300;
 	const MIN_PHP_GD_VERSION = '2.0';
@@ -73,7 +73,7 @@ class CFrontendSetup {
 		}
 
 		// check for deprecated PHP 5.6.0 option 'always_populate_raw_post_data'
-		if (version_compare(PHP_VERSION, '5.6', '>=')) {
+		if (version_compare(PHP_VERSION, '5.6', '>=') && version_compare(PHP_VERSION, '7.0', '<')) {
 			$result[] = $this->checkPhpAlwaysPopulateRawPostData();
 		}
 		$result[] = $this->checkPhpSockets();
@@ -84,6 +84,7 @@ class CFrontendSetup {
 		$result[] = $this->checkPhpLibxml();
 		$result[] = $this->checkPhpXmlWriter();
 		$result[] = $this->checkPhpXmlReader();
+		$result[] = $this->checkPhpLdapModule();
 		$result[] = $this->checkPhpCtype();
 		$result[] = $this->checkPhpSession();
 		$result[] = $this->checkPhpSessionAutoStart();
@@ -117,14 +118,16 @@ class CFrontendSetup {
 	 */
 	public function checkPhpMemoryLimit() {
 		$current = ini_get('memory_limit');
-		$check = ($current == '-1' || str2mem($current) >= self::MIN_PHP_MEMORY_LIMIT);
+		$check = ($current == '-1' || bccomp(str2mem($current), self::MIN_PHP_MEMORY_LIMIT) >= 0);
 
 		return [
 			'name' => _s('PHP option "%1$s"', 'memory_limit'),
 			'current' => $current,
 			'required' => mem2str(self::MIN_PHP_MEMORY_LIMIT),
 			'result' => $check ? self::CHECK_OK : self::CHECK_FATAL,
-			'error' => _s('Minimum required PHP memory limit is %s (configuration option "memory_limit").', mem2str(self::MIN_PHP_MEMORY_LIMIT))
+			'error' => _s('Minimum required PHP memory limit is %s (configuration option "memory_limit").',
+				mem2str(self::MIN_PHP_MEMORY_LIMIT)
+			)
 		];
 	}
 
@@ -140,8 +143,12 @@ class CFrontendSetup {
 			'name' => _s('PHP option "%1$s"', 'post_max_size'),
 			'current' => $current,
 			'required' => mem2str(self::MIN_PHP_POST_MAX_SIZE),
-			'result' => (str2mem($current) >= self::MIN_PHP_POST_MAX_SIZE) ? self::CHECK_OK : self::CHECK_FATAL,
-			'error' => _s('Minimum required size of PHP post is %s (configuration option "post_max_size").', mem2str(self::MIN_PHP_POST_MAX_SIZE))
+			'result' => (bccomp(str2mem($current), self::MIN_PHP_POST_MAX_SIZE) >= 0)
+				? self::CHECK_OK
+				: self::CHECK_FATAL,
+			'error' => _s('Minimum required size of PHP post is %s (configuration option "post_max_size").',
+				mem2str(self::MIN_PHP_POST_MAX_SIZE)
+			)
 		];
 	}
 
@@ -157,8 +164,12 @@ class CFrontendSetup {
 			'name' => _s('PHP option "%1$s"', 'upload_max_filesize'),
 			'current' => $current,
 			'required' => mem2str(self::MIN_PHP_UPLOAD_MAX_FILESIZE),
-			'result' => (str2mem($current) >= self::MIN_PHP_UPLOAD_MAX_FILESIZE) ? self::CHECK_OK : self::CHECK_FATAL,
-			'error' => _s('Minimum required PHP upload filesize is %s (configuration option "upload_max_filesize").', mem2str(self::MIN_PHP_UPLOAD_MAX_FILESIZE))
+			'result' => (bccomp(str2mem($current), self::MIN_PHP_UPLOAD_MAX_FILESIZE) >= 0)
+				? self::CHECK_OK
+				: self::CHECK_FATAL,
+			'error' => _s('Minimum required PHP upload filesize is %s (configuration option "upload_max_filesize").',
+				mem2str(self::MIN_PHP_UPLOAD_MAX_FILESIZE)
+			)
 		];
 	}
 
@@ -248,7 +259,7 @@ class CFrontendSetup {
 			'current' => empty($current) ? _('off') : new CSpan($current),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
-			'error' => _('At least one of MySQL, PostgreSQL, Oracle, SQLite3 or IBM DB2 should be supported.')
+			'error' => _('At least one of MySQL, PostgreSQL, Oracle or IBM DB2 should be supported.')
 		];
 	}
 
@@ -282,11 +293,6 @@ class CFrontendSetup {
 			$allowed_db[ZBX_DB_DB2] = 'IBM DB2';
 		}
 
-		// Semaphore related functions are checked elsewhere. The 'false' is to prevent autoloading of the SQLite3 class.
-		if (class_exists('SQLite3', false) && zbx_is_callable(['ftok', 'sem_acquire', 'sem_get', 'sem_release'])) {
-			$allowed_db[ZBX_DB_SQLITE3] = 'SQLite3';
-		}
-
 		return $allowed_db;
 	}
 
@@ -545,6 +551,23 @@ class CFrontendSetup {
 		];
 	}
 
+	/**
+	 * Checks for PHP LDAP extension.
+	 *
+	 * @return array
+	 */
+	public function checkPhpLdapModule() {
+		$current = (new CLdap())->error !== CLdap::ERR_PHP_EXTENSION;
+
+		return [
+			'name' => _('PHP LDAP'),
+			'current' => $current ? _('on') : _('off'),
+			'required' => null,
+			'result' => $current ? self::CHECK_OK : self::CHECK_WARNING,
+			'error' => _('PHP LDAP extension missing.')
+		];
+	}
+
 	/**
 	 * Checks for PHP ctype extension.
 	 *
