@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,8 +18,11 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-
-require_once dirname(__FILE__).'/include/config.inc.php';
+require_once dirname(__FILE__).'/include/func.inc.php';
+require_once dirname(__FILE__).'/include/defines.inc.php';
+require_once dirname(__FILE__).'/include/classes/json/CJson.php';
+require_once dirname(__FILE__).'/include/classes/user/CWebUser.php';
+require_once dirname(__FILE__).'/include/classes/core/CHttpRequest.php';
 
 $requestType = getRequest('type', PAGE_TYPE_JSON);
 if ($requestType == PAGE_TYPE_JSON) {
@@ -31,6 +34,13 @@ else {
 	$data = $_REQUEST;
 }
 
+if (is_array($data) && array_key_exists('method', $data)
+		&& in_array($data['method'], ['message.settings', 'message.get', 'zabbix.status'])) {
+	CWebUser::disableSessionExtension();
+}
+
+require_once dirname(__FILE__).'/include/config.inc.php';
+
 $page['title'] = 'RPC';
 $page['file'] = 'jsrpc.php';
 $page['type'] = detect_page_type($requestType);
@@ -43,12 +53,13 @@ if (!is_array($data) || !isset($data['method'])
 }
 
 $result = [];
+
 switch ($data['method']) {
-	case 'host.get':
+	case 'search':
 		$result = API::Host()->get([
-			'startSearch' => true,
-			'search' => $data['params']['search'],
-			'output' => ['hostid', 'host', 'name'],
+			'output' => ['hostid', 'name'],
+			'search' => ['name' => $data['params']['search'], 'host' => $data['params']['search']],
+			'searchByAny' => true,
 			'sortfield' => 'name',
 			'limit' => 15
 		]);
@@ -67,7 +78,9 @@ switch ($data['method']) {
 		break;
 
 	case 'message.settings':
-		$result = getMessageSettings();
+		$msgsettings = getMessageSettings();
+		$msgsettings['timeout'] = timeUnitToSeconds($msgsettings['timeout']);
+		$result = $msgsettings;
 		break;
 
 	case 'message.get':
@@ -79,13 +92,14 @@ switch ($data['method']) {
 		}
 
 		// timeout
-		$timeout = time() - $msgsettings['timeout'];
+		$timeout = time() - timeUnitToSeconds($msgsettings['timeout']);
 		$lastMsgTime = 0;
 		if (isset($data['params']['messageLast']['events'])) {
 			$lastMsgTime = $data['params']['messageLast']['events']['time'];
 		}
 
 		$options = [
+			'monitored' => true,
 			'lastChangeSince' => max([$lastMsgTime, $msgsettings['last.clock'], $timeout]),
 			'value' => [TRIGGER_VALUE_TRUE, TRIGGER_VALUE_FALSE],
 			'priority' => array_keys($msgsettings['triggers.severities']),
@@ -94,6 +108,9 @@ switch ($data['method']) {
 		if (!$msgsettings['triggers.recovery']) {
 			$options['value'] = [TRIGGER_VALUE_TRUE];
 		}
+		if (!$msgsettings['show_suppressed']) {
+			$options['suppressed'] = $msgsettings['show_suppressed'];
+		}
 		$events = getLastEvents($options);
 
 		$sortClock = [];
@@ -117,8 +134,16 @@ switch ($data['method']) {
 						$sound = $msgsettings['sounds.'.$trigger['priority']];
 					}
 
-					$url_tr_status = 'tr_status.php?hostid='.$host['hostid'];
-					$url_events = 'events.php?filter_set=1&triggerid='.$event['objectid'].'&source='.EVENT_SOURCE_TRIGGERS;
+					$url_problems = (new CUrl('zabbix.php'))
+						->setArgument('action', 'problem.view')
+						->setArgument('filter_hostids[]', $host['hostid'])
+						->setArgument('filter_set', '1')
+						->getUrl();
+					$url_events = (new CUrl('zabbix.php'))
+						->setArgument('action', 'problem.view')
+						->setArgument('filter_triggerids[]', $event['objectid'])
+						->setArgument('filter_set', '1')
+						->getUrl();
 					$url_tr_events = 'tr_events.php?eventid='.$event['eventid'].'&triggerid='.$event['objectid'];
 
 					$result[$number] = [
@@ -129,7 +154,7 @@ switch ($data['method']) {
 						'priority' => $priority,
 						'sound' => $sound,
 						'severity_style' => getSeverityStyle($trigger['priority'], $event['value'] == TRIGGER_VALUE_TRUE),
-						'title' => $title.' [url='.$url_tr_status.']'.CHtml::encode($host['name']).'[/url]',
+						'title' => $title.' [url='.$url_problems.']'.CHtml::encode($host['name']).'[/url]',
 						'body' => [
 							'[url='.$url_events.']'.CHtml::encode($trigger['description']).'[/url]',
 							'[url='.$url_tr_events.']'.
@@ -165,7 +190,7 @@ switch ($data['method']) {
 		if (!CSession::keyExists('serverCheckResult')
 				|| (CSession::getValue('serverCheckTime') + SERVER_CHECK_INTERVAL) <= time()) {
 			$zabbixServer = new CZabbixServer($ZBX_SERVER, $ZBX_SERVER_PORT, ZBX_SOCKET_TIMEOUT, 0);
-			CSession::setValue('serverCheckResult', $zabbixServer->isRunning());
+			CSession::setValue('serverCheckResult', $zabbixServer->isRunning(CWebUser::getSessionCookie()));
 			CSession::setValue('serverCheckTime', time());
 		}
 
@@ -186,10 +211,30 @@ switch ($data['method']) {
 			if ($data['mode'] == SCREEN_MODE_JS) {
 				$result = $screen;
 			}
-			else {
-				if (is_object($screen)) {
-					$result = $screen->toString();
-				}
+			elseif (is_object($screen)) {
+				$result = $screen->toString();
+			}
+		}
+		break;
+
+	case 'trigger.get':
+		$config = select_config();
+		$result = [];
+
+		$triggers = API::Trigger()->get([
+			'output' => ['triggerid', 'expression', 'priority'],
+			'triggerids' => $data['triggerids'],
+			'limit' => $config['search_limit']
+		]);
+
+		if ($triggers) {
+			CArrayHelper::sort($triggers, [
+				['field' => 'priority', 'order' => ZBX_SORT_DOWN]
+			]);
+
+			foreach ($triggers as $trigger) {
+				$trigger['class_name'] = getSeverityStyle($trigger['priority']);
+				$result[] = $trigger;
 			}
 		}
 		break;
@@ -209,15 +254,23 @@ switch ($data['method']) {
 
 		switch ($data['objectName']) {
 			case 'hostGroup':
-				$hostGroups = API::HostGroup()->get([
-					'editable' => isset($data['editable']) ? $data['editable'] : null,
+				$options = [
+					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
 					'output' => ['groupid', 'name'],
-					'search' => isset($data['search']) ? ['name' => $data['search']] : null,
-					'filter' => isset($data['filter']) ? $data['filter'] : null,
-					'limit' => isset($data['limit']) ? $data['limit'] : null
-				]);
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
+					'limit' => array_key_exists('limit', $data) ? $data['limit'] : null,
+					'real_hosts' => array_key_exists('real_hosts', $data) ? $data['real_hosts'] : null
+				];
+				$hostGroups = API::HostGroup()->get($options);
 
 				if ($hostGroups) {
+					if (array_key_exists('enrich_parent_groups', $data)) {
+						$hostGroups = CPageFilter::enrichParentGroups($hostGroups, [
+							'real_hosts' => null
+						] + $options);
+					}
+
 					CArrayHelper::sort($hostGroups, [
 						['field' => 'name', 'order' => ZBX_SORT_UP]
 					]);
@@ -226,24 +279,20 @@ switch ($data['method']) {
 						$hostGroups = array_slice($hostGroups, 0, $data['limit']);
 					}
 
-					foreach ($hostGroups as $hostGroup) {
-						$result[] = [
-							'id' => $hostGroup['groupid'],
-							'name' => $hostGroup['name']
-						];
-					}
+					$result = CArrayHelper::renameObjectsKeys($hostGroups, ['groupid' => 'id']);
 				}
 				break;
 
 			case 'hosts':
 				$hosts = API::Host()->get([
-					'editable' => isset($data['editable']) ? $data['editable'] : null,
+					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
 					'output' => ['hostid', 'name'],
-					'templated_hosts' => isset($data['templated_hosts']) ? $data['templated_hosts'] : null,
-					'search' => isset($data['search']) ? ['name' => $data['search']] : null,
+					'templated_hosts' => array_key_exists('templated_hosts', $data) ? $data['templated_hosts'] : null,
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'limit' => $config['search_limit']
 				]);
 
+
 				if ($hosts) {
 					CArrayHelper::sort($hosts, [
 						['field' => 'name', 'order' => ZBX_SORT_UP]
@@ -253,10 +302,37 @@ switch ($data['method']) {
 						$hosts = array_slice($hosts, 0, $data['limit']);
 					}
 
-					foreach ($hosts as $host) {
+					$result = CArrayHelper::renameObjectsKeys($hosts, ['hostid' => 'id']);
+				}
+				break;
+
+			case 'items':
+				$items = API::Item()->get([
+					'output' => ['itemid', 'hostid', 'name', 'key_'],
+					'selectHosts' => ['name'],
+					'hostids' => array_key_exists('hostid', $data) ? $data['hostid'] : null,
+					'templated' => array_key_exists('real_hosts', $data) ? false : null,
+					'webitems' => array_key_exists('webitems', $data) ? $data['webitems'] : null,
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
+					'limit' => $config['search_limit']
+				]);
+
+				if ($items) {
+					$items = CMacrosResolverHelper::resolveItemNames($items);
+					CArrayHelper::sort($items, [
+						['field' => 'name_expanded', 'order' => ZBX_SORT_UP]
+					]);
+
+					if (array_key_exists('limit', $data)) {
+						$items = array_slice($items, 0, $data['limit']);
+					}
+
+					foreach ($items as $item) {
 						$result[] = [
-							'id' => $host['hostid'],
-							'name' => $host['name']
+							'id' => $item['itemid'],
+							'name' => $item['name_expanded'],
+							'prefix' => $item['hosts'][0]['name'].NAME_DELIMITER
 						];
 					}
 				}
@@ -264,7 +340,7 @@ switch ($data['method']) {
 
 			case 'templates':
 				$templates = API::Template()->get([
-					'editable' => isset($data['editable']) ? $data['editable'] : null,
+					'editable' => isset($data['editable']) ? $data['editable'] : false,
 					'output' => ['templateid', 'name'],
 					'search' => isset($data['search']) ? ['name' => $data['search']] : null,
 					'limit' => $config['search_limit']
@@ -279,12 +355,25 @@ switch ($data['method']) {
 						$templates = array_slice($templates, 0, $data['limit']);
 					}
 
-					foreach ($templates as $template) {
-						$result[] = [
-							'id' => $template['templateid'],
-							'name' => $template['name']
-						];
+					$result = CArrayHelper::renameObjectsKeys($templates, ['templateid' => 'id']);
+				}
+				break;
+
+			case 'proxies':
+				$proxies = API::Proxy()->get([
+					'output' => ['proxyid', 'host'],
+					'search' => array_key_exists('search', $data) ? ['host' => $data['search']] : null,
+					'limit' => $config['search_limit']
+				]);
+
+				if ($proxies) {
+					CArrayHelper::sort($proxies, ['host']);
+
+					if (isset($data['limit'])) {
+						$proxies = array_slice($proxies, 0, $data['limit']);
 					}
+
+					$result = CArrayHelper::renameObjectsKeys($proxies, ['proxyid' => 'id', 'host' => 'name']);
 				}
 				break;
 
@@ -305,25 +394,38 @@ switch ($data['method']) {
 						$applications = array_slice($applications, 0, $data['limit']);
 					}
 
-					foreach ($applications as $application) {
-						$result[] = [
-							'id' => $application['applicationid'],
-							'name' => $application['name']
-						];
-					}
+					$result = CArrayHelper::renameObjectsKeys($applications, ['applicationid' => 'id']);
 				}
 				break;
 
 			case 'triggers':
+				$host_fields = ['name'];
+				if (array_key_exists('real_hosts', $data) && $data['real_hosts']) {
+					$host_fields[] = 'status';
+				}
+
 				$triggers = API::Trigger()->get([
-					'editable' => isset($data['editable']) ? $data['editable'] : null,
 					'output' => ['triggerid', 'description'],
-					'selectHosts' => ['name'],
+					'selectHosts' => $host_fields,
+					'editable' => isset($data['editable']) ? $data['editable'] : false,
+					'monitored' => isset($data['monitored']) ? $data['monitored'] : null,
 					'search' => isset($data['search']) ? ['description' => $data['search']] : null,
 					'limit' => $config['search_limit']
 				]);
 
 				if ($triggers) {
+					if (array_key_exists('real_hosts', $data) && $data['real_hosts']) {
+						foreach ($triggers as $key => $trigger) {
+							foreach ($triggers[$key]['hosts'] as $host) {
+								if ($host['status'] != HOST_STATUS_MONITORED
+										&& $host['status'] != HOST_STATUS_NOT_MONITORED) {
+									unset($triggers[$key]);
+									break;
+								}
+							}
+						}
+					}
+
 					CArrayHelper::sort($triggers, [
 						['field' => 'description', 'order' => ZBX_SORT_UP]
 					]);
@@ -352,7 +454,6 @@ switch ($data['method']) {
 
 			case 'users':
 				$users = API::User()->get([
-					'editable' => array_key_exists('editable', $data) ? $data['editable'] : null,
 					'output' => ['userid', 'alias', 'name', 'surname'],
 					'search' => array_key_exists('search', $data)
 						? [
@@ -382,6 +483,48 @@ switch ($data['method']) {
 					}
 				}
 				break;
+
+			case 'usersGroups':
+				$groups = API::UserGroup()->get([
+					'output' => ['usrgrpid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'limit' => $config['search_limit']
+				]);
+
+				if ($groups) {
+					CArrayHelper::sort($groups, [
+						['field' => 'name', 'order' => ZBX_SORT_UP]
+					]);
+
+					if (array_key_exists('limit', $data)) {
+						$groups = array_slice($groups, 0, $data['limit']);
+					}
+
+					$result = CArrayHelper::renameObjectsKeys($groups, ['usrgrpid' => 'id']);
+				}
+				break;
+
+			case 'drules':
+				$drules = API::DRule()->get([
+					'output' => ['druleid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'filter' => ['status' => DRULE_STATUS_ACTIVE],
+					'limit' => $config['search_limit']
+				]);
+
+				if ($drules) {
+					CArrayHelper::sort($drules, [
+						['field' => 'name', 'order' => ZBX_SORT_UP]
+					]);
+
+					if (array_key_exists('limit', $data)) {
+						$applications = array_slice($drules, 0, $data['limit']);
+					}
+
+					$result = CArrayHelper::renameObjectsKeys($drules, ['druleid' => 'id']);
+				}
+				break;
+
 		}
 		break;
 
