@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 
 require_once dirname(__FILE__).'/include/config.inc.php';
+require_once dirname(__FILE__).'/include/hostgroups.inc.php';
 require_once dirname(__FILE__).'/include/hosts.inc.php';
 require_once dirname(__FILE__).'/include/screens.inc.php';
 require_once dirname(__FILE__).'/include/forms.inc.php';
@@ -44,8 +45,7 @@ require_once dirname(__FILE__).'/include/page_header.php';
 
 //		VAR						TYPE		OPTIONAL FLAGS			VALIDATION	EXCEPTION
 $fields = [
-	'hosts'				=> [T_ZBX_INT, O_OPT, P_SYS,		DB_ID,	null],
-	'groups'			=> [T_ZBX_INT, O_OPT, P_SYS,		DB_ID,	null],
+	'groups'			=> [T_ZBX_STR, O_OPT, null,			NOT_EMPTY,	'isset({add}) || isset({update})'],
 	'clear_templates'	=> [T_ZBX_INT, O_OPT, P_SYS,		DB_ID,	null],
 	'templates'			=> [T_ZBX_INT, O_OPT, null,		DB_ID,	null],
 	'add_templates'		=> [T_ZBX_INT, O_OPT, null,		DB_ID,	null],
@@ -54,8 +54,6 @@ $fields = [
 	'template_name'		=> [T_ZBX_STR, O_OPT, null,		NOT_EMPTY, 'isset({add}) || isset({update})', _('Template name')],
 	'visiblename'		=> [T_ZBX_STR, O_OPT, null,		null,	'isset({add}) || isset({update})'],
 	'groupid'			=> [T_ZBX_INT, O_OPT, P_SYS,		DB_ID,	null],
-	'twb_groupid'		=> [T_ZBX_INT, O_OPT, P_SYS,		DB_ID,	null],
-	'newgroup'			=> [T_ZBX_STR, O_OPT, null,		null,	null],
 	'description'		=> [T_ZBX_STR, O_OPT, null,		null,	null],
 	'macros'			=> [T_ZBX_STR, O_OPT, P_SYS,		null,	null],
 	'show_inherited_macros' => [T_ZBX_INT, O_OPT, null,	IN([0,1]), null],
@@ -75,6 +73,11 @@ $fields = [
 	'cancel'			=> [T_ZBX_STR, O_OPT, P_SYS,		null,	null],
 	'form'				=> [T_ZBX_STR, O_OPT, P_SYS,		null,	null],
 	'form_refresh'		=> [T_ZBX_INT, O_OPT, null,		null,	null],
+	// filter
+	'filter_set'		=> [T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_rst'		=> [T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_name'		=> [T_ZBX_STR, O_OPT, null,		null,		null],
+	'filter_templates' =>  [T_ZBX_INT, O_OPT, null,		DB_ID,		null],
 	// sort and sortorder
 	'sort'				=> [T_ZBX_STR, O_OPT, P_SYS, IN('"name"'),									null],
 	'sortorder'			=> [T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
@@ -84,11 +87,19 @@ check_fields($fields);
 /*
  * Permissions
  */
-if (getRequest('groupid') && !API::HostGroup()->isWritable([$_REQUEST['groupid']])) {
+if (getRequest('groupid') && !isWritableHostGroups([getRequest('groupid')])) {
 	access_deny();
 }
-if (getRequest('templateid') && !API::Template()->isWritable([$_REQUEST['templateid']])) {
-	access_deny();
+if (getRequest('templateid')) {
+	$templates = API::Template()->get([
+		'output' => [],
+		'templateids' => getRequest('templateid'),
+		'editable' => true
+	]);
+
+	if (!$templates) {
+		access_deny();
+	}
 }
 
 $templateIds = getRequest('templates', []);
@@ -128,8 +139,6 @@ if (isset($_REQUEST['add_template']) && isset($_REQUEST['add_templates'])) {
 	$_REQUEST['templates'] = array_merge($templateIds, $_REQUEST['add_templates']);
 }
 if (hasRequest('unlink') || hasRequest('unlink_and_clear')) {
-	$_REQUEST['clear_templates'] = getRequest('clear_templates', []);
-
 	$unlinkTemplates = [];
 
 	if (hasRequest('unlink') && is_array(getRequest('unlink'))) {
@@ -137,113 +146,113 @@ if (hasRequest('unlink') || hasRequest('unlink_and_clear')) {
 	}
 	elseif (hasRequest('unlink_and_clear') && is_array(getRequest('unlink_and_clear'))) {
 		$unlinkTemplates = array_keys(getRequest('unlink_and_clear'));
-		$_REQUEST['clear_templates'] = array_merge(getRequest('unlink_and_clear'), $unlinkTemplates);
+		$_REQUEST['clear_templates'] = array_merge($unlinkTemplates, getRequest('clear_templates', []));
 	}
 
 	foreach ($unlinkTemplates as $id) {
 		unset($_REQUEST['templates'][array_search($id, $_REQUEST['templates'])]);
 	}
 }
-elseif (isset($_REQUEST['clone']) && isset($_REQUEST['templateid'])) {
-	$_REQUEST['form'] = 'clone';
-	unset($_REQUEST['templateid'], $_REQUEST['hosts']);
-}
-elseif (isset($_REQUEST['full_clone']) && isset($_REQUEST['templateid'])) {
-	$_REQUEST['form'] = 'full_clone';
-	$_REQUEST['hosts'] = [];
+elseif ((hasRequest('clone') || hasRequest('full_clone')) && hasRequest('templateid')) {
+	$_REQUEST['form'] = hasRequest('clone') ? 'clone' : 'full_clone';
+
+	$groups = getRequest('groups', []);
+	$groupids = [];
+
+	// Remove inaccessible groups from request, but leave "new".
+	foreach ($groups as $group) {
+		if (!is_array($group)) {
+			$groupids[] = $group;
+		}
+	}
+
+	if ($groupids) {
+		$groups_allowed = API::HostGroup()->get([
+			'output' => [],
+			'groupids' => $groupids,
+			'editable' => true,
+			'preservekeys' => true
+		]);
+
+		foreach ($groups as $idx => $group) {
+			if (!is_array($group) && !array_key_exists($group, $groups_allowed)) {
+				unset($groups[$idx]);
+			}
+		}
+
+		$_REQUEST['groups'] = $groups;
+	}
+
+	if (hasRequest('clone')) {
+		unset($_REQUEST['templateid']);
+	}
 }
 elseif (hasRequest('add') || hasRequest('update')) {
-	$templateId = getRequest('templateid');
-
 	try {
 		DBstart();
 
-		$templates = getRequest('templates', []);
-		$templateName = getRequest('template_name', '');
-
-		// clone template id
-		$cloneTemplateId = null;
-		$templatesClear = getRequest('clear_templates', []);
+		$templateId = getRequest('templateid', 0);
+		$cloneTemplateId = 0;
 
 		if (getRequest('form') === 'full_clone') {
 			$cloneTemplateId = $templateId;
-			$templateId = null;
+			$templateId = 0;
 		}
 
-		// macros
-		$macros = getRequest('macros', []);
+		if ($templateId == 0) {
+			$messageSuccess = _('Template added');
+			$messageFailed = _('Cannot add template');
+			$auditAction = AUDIT_ACTION_ADD;
+		}
+		else {
+			$messageSuccess = _('Template updated');
+			$messageFailed = _('Cannot update template');
+			$auditAction = AUDIT_ACTION_UPDATE;
+		}
 
-		// groups
+		// Add new group.
 		$groups = getRequest('groups', []);
-		$groups = zbx_toObject($groups, 'groupid');
+		$new_groups = [];
 
-		// create new group
-		$newGroup = getRequest('newgroup');
-
-		if (!zbx_empty($newGroup)) {
-			$result = API::HostGroup()->create([
-				'name' => $newGroup
-			]);
-
-			$newGroup = API::HostGroup()->get([
-				'groupids' => $result['groupids'],
-				'output' => API_OUTPUT_EXTEND
-			]);
-
-			if ($newGroup) {
-				$groups = array_merge($groups, $newGroup);
+		foreach ($groups as $idx => $group) {
+			if (is_array($group) && array_key_exists('new', $group)) {
+				$new_groups[] = ['name' => $group['new']];
+				unset($groups[$idx]);
 			}
-			else {
+		}
+
+		if ($new_groups) {
+			$new_groupid = API::HostGroup()->create($new_groups);
+
+			if (!$new_groupid) {
 				throw new Exception();
 			}
+
+			$groups = array_merge($groups, $new_groupid['groupids']);
 		}
 
 		// linked templates
-		$linkedTemplates = $templates;
+		$linkedTemplates = getRequest('templates', []);
 		$templates = [];
 		foreach ($linkedTemplates as $linkedTemplateId) {
 			$templates[] = ['templateid' => $linkedTemplateId];
 		}
 
+		$templatesClear = getRequest('clear_templates', []);
 		$templatesClear = zbx_toObject($templatesClear, 'templateid');
-
-		// discovered hosts
-		$dbHosts = API::Host()->get([
-			'output' => ['hostid'],
-			'hostids' => getRequest('hosts', []),
-			'templated_hosts' => true,
-			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL]
-		]);
+		$templateName = getRequest('template_name', '');
 
 		// create / update template
 		$template = [
 			'host' => $templateName,
 			'name' => getRequest('visiblename', ''),
-			'groups' => $groups,
+			'groups' => zbx_toObject($groups, 'groupid'),
 			'templates' => $templates,
-			'hosts' => $dbHosts,
-			'macros' => $macros,
+			'macros' => getRequest('macros', []),
 			'description' => getRequest('description', '')
 		];
 
-		if ($templateId) {
-			$template['templateid'] = $templateId;
-			$template['templates_clear'] = $templatesClear;
-
-			$messageSuccess = _('Template updated');
-			$messageFailed = _('Cannot update template');
-			$auditAction = AUDIT_ACTION_UPDATE;
-
-			$result = API::Template()->update($template);
-			if (!$result) {
-				throw new Exception();
-			}
-		}
-		else {
-			$messageSuccess = _('Template added');
-			$messageFailed = _('Cannot add template');
-			$auditAction = AUDIT_ACTION_ADD;
-
+		if ($templateId == 0) {
 			$result = API::Template()->create($template);
 
 			if ($result) {
@@ -253,19 +262,32 @@ elseif (hasRequest('add') || hasRequest('update')) {
 				throw new Exception();
 			}
 		}
+		else {
+			$template['templateid'] = $templateId;
+			$template['templates_clear'] = $templatesClear;
+
+			$result = API::Template()->update($template);
+
+			if (!$result) {
+				throw new Exception();
+			}
+		}
 
 		// full clone
-		if ($templateId && $cloneTemplateId && getRequest('form') === 'full_clone') {
+		if ($cloneTemplateId != 0 && getRequest('form') === 'full_clone') {
 			if (!copyApplications($cloneTemplateId, $templateId)) {
 				throw new Exception();
 			}
 
-			if (!copyItems($cloneTemplateId, $templateId)) {
+			/*
+			 * First copy web scenarios with web items, so that later regular items can use web item as their master
+			 * item.
+			 */
+			if (!copyHttpTests($cloneTemplateId, $templateId)) {
 				throw new Exception();
 			}
 
-			// copy web scenarios
-			if (!copyHttpTests($cloneTemplateId, $templateId)) {
+			if (!copyItems($cloneTemplateId, $templateId)) {
 				throw new Exception();
 			}
 
@@ -408,14 +430,20 @@ elseif (hasRequest('action') && str_in_array(getRequest('action'), ['template.ma
 	if ($result) {
 		uncheckTableRows();
 	}
+	else {
+		$templateids = API::Template()->get([
+			'output' => [],
+			'templateids' => $templates,
+			'editable' => true
+		]);
+		uncheckTableRows(null, zbx_objectValues($templateids, 'templateid'));
+	}
 	show_messages($result, _('Template deleted'), _('Cannot delete template'));
 }
 
 /*
  * Display
  */
-$templateWidget = (new CWidget())->setTitle(_('Templates'));
-
 $pageFilter = new CPageFilter([
 	'config' => [
 		'individual' => 1
@@ -429,21 +457,15 @@ $pageFilter = new CPageFilter([
 $_REQUEST['groupid'] = $pageFilter->groupid;
 
 if (hasRequest('form')) {
-
-	if ($templateId = getRequest('templateid', 0)) {
-		$templateWidget->addItem(get_header_host_table('', $templateId));
-	}
-
 	$data = [
 		'form' => getRequest('form'),
-		'groupId' => getRequest('groupid', 0),
-		'groupIds' => getRequest('groups', []),
+		'templateid' => getRequest('templateid', 0),
 		'show_inherited_macros' => getRequest('show_inherited_macros', 0)
 	];
 
-	if ($templateId) {
+	if ($data['templateid'] != 0) {
 		$dbTemplates = API::Template()->get([
-			'templateids' => $templateId,
+			'templateids' => $data['templateid'],
 			'selectGroups' => API_OUTPUT_EXTEND,
 			'selectParentTemplates' => ['templateid', 'name'],
 			'selectMacros' => API_OUTPUT_EXTEND,
@@ -461,7 +483,7 @@ if (hasRequest('form')) {
 	}
 
 	// description
-	$data['description'] = ($templateId && !hasRequest('form_refresh'))
+	$data['description'] = ($data['templateid'] != 0 && !hasRequest('form_refresh'))
 		? $data['dbTemplate']['description']
 		: getRequest('description');
 
@@ -473,92 +495,81 @@ if (hasRequest('form')) {
 		'templateids' => $templateIds,
 		'preservekeys' => true
 	]);
+
+	$data['writable_templates'] = API::Template()->get([
+		'output' => ['templateid'],
+		'templateids' => $templateIds,
+		'editable' => true,
+		'preservekeys' => true
+	]);
+
 	CArrayHelper::sort($data['linkedTemplates'], ['name']);
 
-	// Get user allowed host groups and sort them by name.
-	$data['groupsAllowed'] = API::HostGroup()->get([
-		'output' => ['groupid', 'name'],
-		'editable' => true,
-		'preservekeys' => true
-	]);
-	CArrayHelper::sort($data['groupsAllowed'], ['name']);
+	$groups = [];
 
-	// Get other host groups that user has also read permissions and sort by name.
-	$data['groupsAll'] = API::HostGroup()->get([
-		'output' => ['groupid', 'name'],
-		'preservekeys' => true
-	]);
-	CArrayHelper::sort($data['groupsAll'], ['name']);
-
-	// "Other | group" tweenbox selector for hosts and templates
-	$data['twb_groupid'] = getRequest('twb_groupid', 0);
-	if ($data['twb_groupid'] == 0) {
-		$group = reset($data['groupsAllowed']);
-		$data['twb_groupid'] = $group['groupid'];
-	}
-
-	// Get allowed hosts from selected twb_groupid combobox.
-	$data['hostsAllowedToAdd'] = API::Host()->get([
-		'output' => ['hostid', 'name'],
-		'groupids' => $data['twb_groupid'],
-		'templated_hosts' => true,
-		'editable' => true,
-		'preservekeys' => true,
-		'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL]
-	]);
-	CArrayHelper::sort($data['hostsAllowedToAdd'], ['name']);
-
-	if ($templateId != 0 && !hasRequest('form_refresh')) {
-		$data['groupIds'] = zbx_objectValues($data['dbTemplate']['groups'], 'groupid');
-
-		// Get template hosts from DB.
-		$hostIdsLinkedTo = API::Host()->get([
-			'output' => ['hostid'],
-			'templateids' => $templateId,
-			'templated_hosts' => true,
-			'preservekeys' => true
-		]);
-		$hostIdsLinkedTo = array_keys($hostIdsLinkedTo);
+	if (!hasRequest('form_refresh')) {
+		if ($data['templateid'] != 0) {
+			$groups = zbx_objectValues($data['dbTemplate']['groups'], 'groupid');
+		}
+		elseif (getRequest('groupid', 0) != 0) {
+			$groups[] = getRequest('groupid');
+		}
 	}
 	else {
-		if (!hasRequest('form_refresh') && $data['groupId'] != 0 && !$data['groupIds']) {
-			$data['groupIds'][] = $data['groupId'];
+		$groups = getRequest('groups', []);
+	}
+
+	$groupids = [];
+
+	foreach ($groups as $group) {
+		if (is_array($group) && array_key_exists('new', $group)) {
+			continue;
 		}
-		$hostIdsLinkedTo = getRequest('hosts', []);
+
+		$groupids[] = $group;
 	}
 
-	if ($data['groupIds']) {
-		$data['groupIds'] = array_combine($data['groupIds'], $data['groupIds']);
+	// Groups with R and RW permissions.
+	$groups_all = $groupids
+		? API::HostGroup()->get([
+			'output' => ['name'],
+			'groupids' => $groupids,
+			'preservekeys' => true
+		])
+		: [];
+
+	// Groups with RW permissions.
+	$groups_rw = $groupids && (CWebUser::getType() != USER_TYPE_SUPER_ADMIN)
+		? API::HostGroup()->get([
+			'output' => [],
+			'groupids' => $groupids,
+			'editable' => true,
+			'preservekeys' => true
+		])
+		: [];
+
+	$data['groups_ms'] = [];
+
+	// Prepare data for multiselect.
+	foreach ($groups as $group) {
+		if (is_array($group) && array_key_exists('new', $group)) {
+			$data['groups_ms'][] = [
+				'id' => $group['new'],
+				'name' => $group['new'].' ('._x('new', 'new element in multiselect').')',
+				'isNew' => true
+			];
+		}
+		elseif (array_key_exists($group, $groups_all)) {
+			$data['groups_ms'][] = [
+				'id' => $group,
+				'name' => $groups_all[$group]['name'],
+				'disabled' => (CWebUser::getType() != USER_TYPE_SUPER_ADMIN) && !array_key_exists($group, $groups_rw)
+			];
+		}
 	}
+	CArrayHelper::sort($data['groups_ms'], ['name']);
 
-	if ($hostIdsLinkedTo) {
-		$hostIdsLinkedTo = array_combine($hostIdsLinkedTo, $hostIdsLinkedTo);
-	}
-
-	// Select allowed selected hosts.
-	$data['hostsAllowed'] = API::Host()->get([
-		'output' => ['hostid', 'name', 'flags'],
-		'hostids' => $hostIdsLinkedTo,
-		'templated_hosts' => true,
-		'editable' => true,
-		'preservekeys' => true,
-		'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL]
-	]);
-	CArrayHelper::sort($data['hostsAllowed'], ['name']);
-
-	// Select selected hosts including read only.
-	$data['hostsAll'] = API::Host()->get([
-		'output' => ['hostid', 'name', 'flags'],
-		'hostids' => $hostIdsLinkedTo,
-		'templated_hosts' => true
-	]);
-	CArrayHelper::sort($data['hostsAll'], ['name']);
-
-	$data['hostIdsLinkedTo'] = $hostIdsLinkedTo;
-	$data['templateId'] = $templateId;
-
-	$templateForm = new CView('configuration.template.edit', $data);
-	$templateWidget->addItem($templateForm->render());
+	$view = new CView('configuration.template.edit', $data);
 }
 else {
 	$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'name'));
@@ -567,61 +578,56 @@ else {
 	CProfile::update('web.'.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
 	CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
 
+	// filter
+	if (hasRequest('filter_set')) {
+		CProfile::update('web.templates.filter_name', getRequest('filter_name', ''), PROFILE_TYPE_STR);
+		CProfile::updateArray('web.templates.filter_templates', getRequest('filter_templates', []), PROFILE_TYPE_ID);
+	}
+	elseif (hasRequest('filter_rst')) {
+		CProfile::delete('web.templates.filter_name');
+		CProfile::deleteIdx('web.templates.filter_templates');
+	}
+
+	$filter = [
+		'name' => CProfile::get('web.templates.filter_name', ''),
+		'templates' => CProfile::getArray('web.templates.filter_templates', null)
+	];
+
 	$config = select_config();
 
-	$controls = (new CList())
-		->addItem([_('Group'), SPACE, $pageFilter->getGroupsCB()])
-		->addItem(new CSubmit('form', _('Create template')))
-		->addItem(
-			(new CButton('form', _('Import')))
-				->onClick('redirect("conf.import.php?rules_preset=template")')
-		);
-	$frmForm = (new CForm('get'))
-		->cleanItems()
-		->addItem($controls);
-
-	$templateWidget->setControls($frmForm);
-
-	$form = (new CForm())->setName('templates');
-
-	$table = (new CTableInfo())
-		->setHeader([
-			(new CColHeader(
-				(new CCheckBox('all_templates'))->onClick("checkAll('".$form->getName()."', 'all_templates', 'templates');")
-			))->addClass(ZBX_STYLE_CELL_WIDTH),
-			make_sorting_header(_('Templates'), 'name', $sortField, $sortOrder),
-			_('Applications'),
-			_('Items'),
-			_('Triggers'),
-			_('Graphs'),
-			_('Screens'),
-			_('Discovery'),
-			_('Web'),
-			_('Linked templates'),
-			_('Linked to')
-		]);
-
 	// get templates
 	$templates = [];
 
+	$filter['templates'] = $filter['templates']
+		? CArrayHelper::renameObjectsKeys(API::Template()->get([
+			'output' => ['templateid', 'name'],
+			'templateids' => $filter['templates'],
+			'preservekeys' => true
+		]), ['templateid' => 'id'])
+		: [];
+
 	if ($pageFilter->groupsSelected) {
 		$templates = API::Template()->get([
 			'output' => ['templateid', $sortField],
-			'groupids' => ($pageFilter->groupid > 0) ? $pageFilter->groupid : null,
+			'search' => [
+				'name' => ($filter['name'] === '') ? null : $filter['name']
+			],
+			'parentTemplateids' => $filter['templates'] ? array_keys($filter['templates']) : null,
+			'groupids' => $pageFilter->groupids,
 			'editable' => true,
 			'sortfield' => $sortField,
 			'limit' => $config['search_limit'] + 1
 		]);
 	}
-
-	// sorting && paging
 	order_result($templates, $sortField, $sortOrder);
-	$paging = getPagingLine($templates, $sortOrder);
+
+	$url = (new CUrl('templates.php'))
+		->setArgument('groupid', getRequest('groupid', 0));
+
+	$paging = getPagingLine($templates, $sortOrder, $url);
 
 	$templates = API::Template()->get([
-		'templateids' => zbx_objectValues($templates, 'templateid'),
-		'editable' => true,
-		'output' => ['name', 'proxy_hostid'],
+		'output' => ['templateid', 'name'],
 		'selectHosts' => ['hostid', 'name', 'status'],
 		'selectTemplates' => ['templateid', 'name', 'status'],
 		'selectParentTemplates' => ['templateid', 'name', 'status'],
@@ -632,134 +638,69 @@ else {
 		'selectDiscoveries' => API_OUTPUT_COUNT,
 		'selectScreens' => API_OUTPUT_COUNT,
 		'selectHttpTests' => API_OUTPUT_COUNT,
-		'nopermissions' => true
+		'templateids' => zbx_objectValues($templates, 'templateid'),
+		'editable' => true
 	]);
 
 	order_result($templates, $sortField, $sortOrder);
 
+	// Select writable templates:
+	$linked_template_ids = [];
+	$writable_templates = [];
+	$linked_hosts_ids = [];
+	$writable_hosts = [];
 	foreach ($templates as $template) {
-		$templatesOutput = [];
+		$linked_template_ids = array_merge(
+			$linked_template_ids,
+			zbx_objectValues($template['parentTemplates'], 'templateid'),
+			zbx_objectValues($template['templates'], 'templateid'),
+			zbx_objectValues($template['hosts'], 'hostid')
+		);
 
-		if ($template['proxy_hostid']) {
-			$proxy = get_host_by_hostid($template['proxy_hostid']);
-
-			$templatesOutput[] = $proxy['host'].NAME_DELIMITER;
-		}
-
-		$templatesOutput[] = new CLink($template['name'], 'templates.php?form=update&templateid='.$template['templateid'].url_param('groupid'));
-
-		$linkedTemplatesOutput = [];
-		$linkedToOutput = [];
-
-		$i = 0;
-
-		order_result($template['parentTemplates'], 'name');
-
-		foreach ($template['parentTemplates'] as $parentTemplate) {
-			$i++;
-
-			if ($i > $config['max_in_table']) {
-				$linkedTemplatesOutput[] = ' &hellip;';
-
-				break;
-			}
-
-			if ($linkedTemplatesOutput) {
-				$linkedTemplatesOutput[] = ', ';
-			}
-
-			$url = 'templates.php?form=update&templateid='.$parentTemplate['templateid'].url_param('groupid');
-
-			$linkedTemplatesOutput[] = (new CLink($parentTemplate['name'], $url))
-				->addClass(ZBX_STYLE_LINK_ALT)
-				->addClass(ZBX_STYLE_GREY);
-		}
-
-		$i = 0;
-
-		$linkedToObjects = array_merge($template['hosts'], $template['templates']);
-		order_result($linkedToObjects, 'name');
-
-		foreach ($linkedToObjects as $linkedToObject) {
-			$i++;
-
-			if ($i > $config['max_in_table']) {
-				$linkedToOutput[] = ' &hellip;';
-
-				break;
-			}
-
-			if ($linkedToOutput) {
-				$linkedToOutput[] = ', ';
-			}
-
-			if ($linkedToObject['status'] == HOST_STATUS_TEMPLATE) {
-				$url = 'templates.php?form=update&templateid='.$linkedToObject['templateid'].url_param('groupid');
-			}
-			else {
-				$url = 'hosts.php?form=update&hostid='.$linkedToObject['hostid'].url_param('groupid');
-			}
-
-			$link = (new CLink($linkedToObject['name'], $url))->addClass(ZBX_STYLE_LINK_ALT);
-
-			if ($linkedToObject['status'] == HOST_STATUS_TEMPLATE) {
-				$link->addClass(ZBX_STYLE_GREY);
-			}
-			else {
-				$link->addClass($linkedToObject['status'] == HOST_STATUS_MONITORED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED);
-			}
-
-			$linkedToOutput[] = $link;
-		}
-
-		$table->addRow([
-			new CCheckBox('templates['.$template['templateid'].']', $template['templateid']),
-			(new CCol($templatesOutput))->addClass(ZBX_STYLE_NOWRAP),
-			[
-				new CLink(_('Applications'), 'applications.php?hostid='.$template['templateid'].url_param('groupid')),
-				CViewHelper::showNum($template['applications'])
-			],
-			[
-				new CLink(_('Items'), 'items.php?filter_set=1&hostid='.$template['templateid'].url_param('groupid')),
-				CViewHelper::showNum($template['items'])
-			],
-			[
-				new CLink(_('Triggers'), 'triggers.php?hostid='.$template['templateid'].url_param('groupid')),
-				CViewHelper::showNum($template['triggers'])
-			],
-			[
-				new CLink(_('Graphs'), 'graphs.php?hostid='.$template['templateid'].url_param('groupid')),
-				CViewHelper::showNum($template['graphs'])
-			],
-			[
-				new CLink(_('Screens'), 'screenconf.php?templateid='.$template['templateid']),
-				CViewHelper::showNum($template['screens'])
-			],
-			[
-				new CLink(_('Discovery'), 'host_discovery.php?hostid='.$template['templateid']),
-				CViewHelper::showNum($template['discoveries'])
-			],
-			[
-				new CLink(_('Web'), 'httpconf.php?hostid='.$template['templateid'].url_param('groupid')),
-				CViewHelper::showNum($template['httpTests'])
-			],
-			$linkedTemplatesOutput,
-			$linkedToOutput
+		$linked_hosts_ids = array_merge(
+			$linked_hosts_ids,
+			zbx_objectValues($template['hosts'], 'hostid')
+		);
+	}
+	if ($linked_template_ids) {
+		$linked_template_ids = array_unique($linked_template_ids);
+		$writable_templates = API::Template()->get([
+			'output' => ['templateid'],
+			'templateids' => $linked_template_ids,
+			'editable' => true,
+			'preservekeys' => true
+		]);
+	}
+	if ($linked_hosts_ids) {
+		$linked_hosts_ids = array_unique($linked_hosts_ids);
+		$writable_hosts = API::Host()->get([
+			'output' => ['hostid'],
+			'hostsids' => $linked_hosts_ids,
+			'editable' => true,
+			'preservekeys' => true
 		]);
 	}
 
-	$footer = new CActionButtonList('action', 'templates', [
-		'template.export' => ['name' => _('Export')],
-		'template.massdelete' => ['name' => _('Delete'), 'confirm' => _('Delete selected templates?')],
-		'template.massdeleteclear' => ['name' => _('Delete and clear'),
-			'confirm' => _('Delete and clear selected templates? (Warning: all linked hosts will be cleared!)')
-		]
-	]);
+	$data = [
+		'pageFilter' => $pageFilter,
+		'templates' => $templates,
+		'paging' => $paging,
+		'filter' => $filter,
+		'sortField' => $sortField,
+		'sortOrder' => $sortOrder,
+		'config' => [
+			'max_in_table' => $config['max_in_table']
+		],
+		'writable_templates' => $writable_templates,
+		'writable_hosts' => $writable_hosts,
+		'profileIdx' => 'web.templates.filter',
+		'active_tab' => CProfile::get('web.templates.filter.active', 1)
+	];
 
-	$form->addItem([$table, $paging, $footer]);
-	$templateWidget->addItem($form);
+	$view = new CView('configuration.template.list', $data);
 }
 
-$templateWidget->show();
+$view->render();
+$view->show();
 
 require_once dirname(__FILE__).'/include/page_footer.php';
