@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -92,12 +92,14 @@ struct	_DC_TRIGGER;
 #define HOST_IPMI_USERNAME_LEN_MAX	(HOST_IPMI_USERNAME_LEN + 1)
 #define HOST_IPMI_PASSWORD_LEN		20
 #define HOST_IPMI_PASSWORD_LEN_MAX	(HOST_IPMI_PASSWORD_LEN + 1)
+#define HOST_PROXY_ADDRESS_LEN		255
+#define HOST_PROXY_ADDRESS_LEN_MAX	(HOST_PROXY_ADDRESS_LEN + 1)
 
-#define INTERFACE_DNS_LEN		64
+#define INTERFACE_DNS_LEN		255
 #define INTERFACE_DNS_LEN_MAX		(INTERFACE_DNS_LEN + 1)
 #define INTERFACE_IP_LEN		64
 #define INTERFACE_IP_LEN_MAX		(INTERFACE_IP_LEN + 1)
-#define INTERFACE_ADDR_LEN		64	/* MAX(INTERFACE_DNS_LEN,INTERFACE_IP_LEN) */
+#define INTERFACE_ADDR_LEN		255	/* MAX(INTERFACE_DNS_LEN,INTERFACE_IP_LEN) */
 #define INTERFACE_ADDR_LEN_MAX		(INTERFACE_ADDR_LEN + 1)
 #define INTERFACE_PORT_LEN		64
 #define INTERFACE_PORT_LEN_MAX		(INTERFACE_PORT_LEN + 1)
@@ -138,12 +140,32 @@ struct	_DC_TRIGGER;
 #define ITEM_PRIVATEKEY_LEN_MAX		(ITEM_PRIVATEKEY_LEN + 1)
 #define ITEM_JMX_ENDPOINT_LEN		255
 #define ITEM_JMX_ENDPOINT_LEN_MAX	(ITEM_JMX_ENDPOINT_LEN + 1)
+#define ITEM_TIMEOUT_LEN		255
+#define ITEM_TIMEOUT_LEN_MAX		(ITEM_TIMEOUT_LEN + 1)
+#define ITEM_URL_LEN			2048
+#define ITEM_URL_LEN_MAX		(ITEM_URL_LEN + 1)
+#define ITEM_QUERY_FIELDS_LEN		2048
+#define ITEM_QUERY_FIELDS_LEN_MAX	(ITEM_QUERY_FIELDS_LEN + 1)
+#define ITEM_STATUS_CODES_LEN		255
+#define ITEM_STATUS_CODES_LEN_MAX	(ITEM_STATUS_CODES_LEN + 1)
+#define ITEM_HTTP_PROXY_LEN		255
+#define ITEM_HTTP_PROXY_LEN_MAX		(ITEM_HTTP_PROXY_LEN + 1)
+#define ITEM_SSL_KEY_PASSWORD_LEN	64
+#define ITEM_SSL_KEY_PASSWORD_LEN_MAX	(ITEM_SSL_KEY_PASSWORD_LEN + 1)
+#define ITEM_SSL_CERT_FILE_LEN		255
+#define ITEM_SSL_CERT_FILE_LEN_MAX	(ITEM_SSL_CERT_FILE_LEN + 1)
+#define ITEM_SSL_KEY_FILE_LEN		255
+#define ITEM_SSL_KEY_FILE_LEN_MAX	(ITEM_SSL_KEY_FILE_LEN + 1)
 #if defined(HAVE_IBM_DB2) || defined(HAVE_ORACLE)
 #	define ITEM_PARAM_LEN		2048
 #	define ITEM_DESCRIPTION_LEN	2048
+#	define ITEM_POSTS_LEN		2048
+#	define ITEM_HEADERS_LEN		2048
 #else
 #	define ITEM_PARAM_LEN		65535
 #	define ITEM_DESCRIPTION_LEN	65535
+#	define ITEM_POSTS_LEN		65535
+#	define ITEM_HEADERS_LEN		65535
 #endif
 
 #define HISTORY_STR_VALUE_LEN		255
@@ -175,6 +197,10 @@ struct	_DC_TRIGGER;
 
 #define ITEM_PREPROC_PARAMS_LEN		255
 
+#define EVENT_NAME_LEN			2048
+
+#define FUNCTION_PARAM_LEN		255
+
 #define ZBX_SQL_ITEM_FIELDS	"i.itemid,i.key_,h.host,i.type,i.history,i.hostid,i.value_type,i.delta,"	\
 				"i.units,i.multiplier,i.formula,i.state,i.valuemapid,i.trends,i.data_type"
 #define ZBX_SQL_ITEM_TABLES	"hosts h,items i"
@@ -183,23 +209,37 @@ struct	_DC_TRIGGER;
 #define ZBX_SQL_ITEM_SELECT	ZBX_SQL_ITEM_FIELDS " from " ZBX_SQL_ITEM_TABLES
 
 #ifdef HAVE_ORACLE
-#define	DBbegin_multiple_update(sql, sql_alloc, sql_offset)	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, "begin\n")
-#define	DBend_multiple_update(sql, sql_alloc, sql_offset)	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, "end;")
+#	define ZBX_PLSQL_BEGIN	"begin\n"
+#	define ZBX_PLSQL_END	"end;"
+#	define	DBbegin_multiple_update(sql, sql_alloc, sql_offset)			\
+			zbx_strcpy_alloc(sql, sql_alloc, sql_offset, ZBX_PLSQL_BEGIN)
+#	define	DBend_multiple_update(sql, sql_alloc, sql_offset)			\
+			zbx_strcpy_alloc(sql, sql_alloc, sql_offset, ZBX_PLSQL_END)
+#	if 0 == ZBX_MAX_OVERFLOW_SQL_SIZE
+#		define	ZBX_SQL_EXEC_FROM	ZBX_CONST_STRLEN(ZBX_PLSQL_BEGIN)
+#	else
+#		define	ZBX_SQL_EXEC_FROM	0
+#	endif
+
+#	define	ZBX_SQL_STRCMP		"%s%s%s"
+#	define	ZBX_SQL_STRVAL_EQ(str)				\
+			'\0' != *str ? "='"  : "",		\
+			'\0' != *str ? str   : " is null",	\
+			'\0' != *str ? "'"   : ""
+#	define	ZBX_SQL_STRVAL_NE(str)				\
+			'\0' != *str ? "<>'" : "",		\
+			'\0' != *str ? str   : " is not null",	\
+			'\0' != *str ? "'"   : ""
 
-#define	ZBX_SQL_STRCMP		"%s%s%s"
-#define	ZBX_SQL_STRVAL_EQ(str)	'\0' != *str ? "='"  : "",		\
-				'\0' != *str ? str   : " is null",	\
-				'\0' != *str ? "'"   : ""
-#define	ZBX_SQL_STRVAL_NE(str)	'\0' != *str ? "<>'" : "",		\
-				'\0' != *str ? str   : " is not null",	\
-				'\0' != *str ? "'"   : ""
 #else
-#define	DBbegin_multiple_update(sql, sql_alloc, sql_offset)
-#define	DBend_multiple_update(sql, sql_alloc, sql_offset)
+#	define	DBbegin_multiple_update(sql, sql_alloc, sql_offset)	do {} while (0)
+#	define	DBend_multiple_update(sql, sql_alloc, sql_offset)	do {} while (0)
 
-#define	ZBX_SQL_STRCMP		"%s'%s'"
-#define	ZBX_SQL_STRVAL_EQ(str)	"=", str
-#define	ZBX_SQL_STRVAL_NE(str)	"<>", str
+#	define	ZBX_SQL_EXEC_FROM	0
+
+#	define	ZBX_SQL_STRCMP		"%s'%s'"
+#	define	ZBX_SQL_STRVAL_EQ(str)	"=", str
+#	define	ZBX_SQL_STRVAL_NE(str)	"<>", str
 #endif
 
 #define ZBX_SQL_NULLCMP(f1, f2)	"((" f1 " is null and " f2 " is null) or " f1 "=" f2 ")"
@@ -278,12 +318,15 @@ typedef struct
 	zbx_uint64_t		eventid;
 	DB_TRIGGER		trigger;
 	zbx_uint64_t		objectid;
+	char			*name;
 	int			source;
 	int			object;
 	int			clock;
 	int			value;
 	int			acknowledged;
 	int			ns;
+	int			severity;
+	unsigned char		suppressed;
 
 	zbx_vector_ptr_t	tags;
 
@@ -323,7 +366,7 @@ typedef struct
 	char		*value2;
 	int		condition_result;
 	unsigned char	conditiontype;
-	unsigned char	operator;
+	unsigned char	op;
 }
 DB_CONDITION;
 
@@ -413,7 +456,7 @@ typedef struct
 	char		*ack_longdata;
 	int		esc_period;
 	unsigned char	eventsource;
-	unsigned char	maintenance_mode;
+	unsigned char	pause_suppressed;
 	unsigned char	recovery;
 	unsigned char	status;
 }
@@ -425,6 +468,9 @@ typedef struct
 	zbx_uint64_t	userid;
 	char		*message;
 	int		clock;
+	int		action;
+	int		old_severity;
+	int		new_severity;
 }
 DB_ACKNOWLEDGE;
 
@@ -436,35 +482,18 @@ void	DBclose(void);
 
 #ifdef HAVE_ORACLE
 void	DBstatement_prepare(const char *sql);
-int	DBstatement_execute();
 #endif
-#ifdef HAVE___VA_ARGS__
-#	define DBexecute(fmt, ...) __zbx_DBexecute(ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
-#	define DBexecute_once(fmt, ...) __zbx_DBexecute_once(ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
-#else
-#	define DBexecute __zbx_DBexecute
-#	define DBexecute_once __zbx_DBexecute_once
-#endif
-int	__zbx_DBexecute(const char *fmt, ...);
-int	__zbx_DBexecute_once(const char *fmt, ...);
-
-#ifdef HAVE___VA_ARGS__
-#	define DBselect_once(fmt, ...)	__zbx_DBselect_once(ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
-#	define DBselect(fmt, ...)	__zbx_DBselect(ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
-#else
-#	define DBselect_once	__zbx_DBselect_once
-#	define DBselect		__zbx_DBselect
-#endif
-DB_RESULT	__zbx_DBselect_once(const char *fmt, ...);
-DB_RESULT	__zbx_DBselect(const char *fmt, ...);
-
+int		DBexecute(const char *fmt, ...) __zbx_attr_format_printf(1, 2);
+int		DBexecute_once(const char *fmt, ...) __zbx_attr_format_printf(1, 2);
+DB_RESULT	DBselect_once(const char *fmt, ...) __zbx_attr_format_printf(1, 2);
+DB_RESULT	DBselect(const char *fmt, ...) __zbx_attr_format_printf(1, 2);
 DB_RESULT	DBselectN(const char *query, int n);
 DB_ROW		DBfetch(DB_RESULT result);
 int		DBis_null(const char *field);
 void		DBbegin(void);
-void		DBcommit(void);
+int		DBcommit(void);
 void		DBrollback(void);
-void		DBend(int ret);
+int		DBend(int ret);
 
 const ZBX_TABLE	*DBget_table(const char *tablename);
 const ZBX_FIELD	*DBget_field(const ZBX_TABLE *table, const char *fieldname);
@@ -520,7 +549,7 @@ typedef struct
 zbx_trigger_diff_t;
 
 void	zbx_process_triggers(zbx_vector_ptr_t *triggers, zbx_vector_ptr_t *diffs);
-void	zbx_save_trigger_changes(const zbx_vector_ptr_t *diffs);
+void	zbx_db_save_trigger_changes(const zbx_vector_ptr_t *diffs);
 void	zbx_trigger_diff_free(zbx_trigger_diff_t *diff);
 void	zbx_append_trigger_diff(zbx_vector_ptr_t *trigger_diff, zbx_uint64_t triggerid, unsigned char priority,
 		zbx_uint64_t flags, unsigned char value, unsigned char state, int lastchange, const char *error);
@@ -534,7 +563,6 @@ char	*DBdyn_escape_string_len(const char *src, size_t length);
 char	*DBdyn_escape_like_pattern(const char *src);
 
 zbx_uint64_t	DBadd_host(char *server, int port, int status, int useip, char *ip, int disable_until, int available);
-int	DBhost_exists(char *server);
 int	DBadd_templates_to_host(int hostid, int host_templateid);
 
 int	DBadd_template_linkage(int hostid, int templateid, int items, int triggers, int graphs);
@@ -695,12 +723,12 @@ typedef struct
 	const char	*error;
 
 	zbx_uint64_t	flags;
-#define ZBX_FLAGS_ITEM_DIFF_UNSET			0x0000
-#define ZBX_FLAGS_ITEM_DIFF_UPDATE_STATE		0x0001
-#define ZBX_FLAGS_ITEM_DIFF_UPDATE_ERROR		0x0002
-#define ZBX_FLAGS_ITEM_DIFF_UPDATE_MTIME		0x0004
-#define ZBX_FLAGS_ITEM_DIFF_UPDATE_LASTLOGSIZE		0x0008
-#define ZBX_FLAGS_ITEM_DIFF_UPDATE_LASTCLOCK		0x1000
+#define ZBX_FLAGS_ITEM_DIFF_UNSET			__UINT64_C(0x0000)
+#define ZBX_FLAGS_ITEM_DIFF_UPDATE_STATE		__UINT64_C(0x0001)
+#define ZBX_FLAGS_ITEM_DIFF_UPDATE_ERROR		__UINT64_C(0x0002)
+#define ZBX_FLAGS_ITEM_DIFF_UPDATE_MTIME		__UINT64_C(0x0004)
+#define ZBX_FLAGS_ITEM_DIFF_UPDATE_LASTLOGSIZE		__UINT64_C(0x0008)
+#define ZBX_FLAGS_ITEM_DIFF_UPDATE_LASTCLOCK		__UINT64_C(0x1000)
 #define ZBX_FLAGS_ITEM_DIFF_UPDATE_DB			\
 	(ZBX_FLAGS_ITEM_DIFF_UPDATE_STATE | ZBX_FLAGS_ITEM_DIFF_UPDATE_ERROR |\
 	ZBX_FLAGS_ITEM_DIFF_UPDATE_MTIME | ZBX_FLAGS_ITEM_DIFF_UPDATE_LASTLOGSIZE)
@@ -714,5 +742,28 @@ void	zbx_db_free_event(DB_EVENT *event);
 void	zbx_db_get_eventid_r_eventid_pairs(zbx_vector_uint64_t *eventids, zbx_vector_uint64_pair_t *event_pairs,
 		zbx_vector_uint64_t *r_eventids);
 
+void	zbx_db_trigger_clean(DB_TRIGGER *trigger);
+
+
+typedef struct
+{
+	zbx_uint64_t	hostid;
+	unsigned char	compress;
+	int		version;
+	int		lastaccess;
+
+#define ZBX_FLAGS_PROXY_DIFF_UNSET				__UINT64_C(0x0000)
+#define ZBX_FLAGS_PROXY_DIFF_UPDATE_COMPRESS			__UINT64_C(0x0001)
+#define ZBX_FLAGS_PROXY_DIFF_UPDATE_VERSION			__UINT64_C(0x0002)
+#define ZBX_FLAGS_PROXY_DIFF_UPDATE_LASTACCESS			__UINT64_C(0x0004)
+#define ZBX_FLAGS_PROXY_DIFF_UPDATE (			\
+		ZBX_FLAGS_PROXY_DIFF_UPDATE_COMPRESS |	\
+		ZBX_FLAGS_PROXY_DIFF_UPDATE_VERSION | 	\
+		ZBX_FLAGS_PROXY_DIFF_UPDATE_LASTACCESS)
+	zbx_uint64_t	flags;
+}
+zbx_proxy_diff_t;
+
+int	zbx_db_lock_maintenanceids(zbx_vector_uint64_t *maintenanceids);
 
 #endif
