@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -187,7 +187,7 @@ static zbx_oracle_column_type_t	zbx_oracle_column_type(unsigned char field_type)
 
 static void	DBfield_definition_string(char **sql, size_t *sql_alloc, size_t *sql_offset, const ZBX_FIELD *field)
 {
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s ", field->name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, ZBX_FS_SQL_NAME " ", field->name);
 	DBfield_type_string(sql, sql_alloc, sql_offset, field);
 	if (NULL != field->default_value)
 	{
@@ -251,6 +251,18 @@ static void	DBcreate_table_sql(char **sql, size_t *sql_alloc, size_t *sql_offset
 	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, "\n)" ZBX_DB_TABLE_OPTIONS);
 }
 
+static void	DBrename_table_sql(char **sql, size_t *sql_alloc, size_t *sql_offset, const char *table_name,
+		const char *new_name)
+{
+#ifdef HAVE_IBM_DB2
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "rename table " ZBX_FS_SQL_NAME " to " ZBX_FS_SQL_NAME,
+			table_name, new_name);
+#else
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table " ZBX_FS_SQL_NAME " rename to " ZBX_FS_SQL_NAME,
+			table_name, new_name);
+#endif
+}
+
 static void	DBdrop_table_sql(char **sql, size_t *sql_alloc, size_t *sql_offset, const char *table_name)
 {
 	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "drop table %s", table_name);
@@ -273,7 +285,8 @@ static void	DBset_default_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
 static void	DBmodify_field_type_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
 		const char *table_name, const ZBX_FIELD *field)
 {
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table %s" ZBX_DB_ALTER_COLUMN " ", table_name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table " ZBX_FS_SQL_NAME ZBX_DB_ALTER_COLUMN " ",
+			table_name);
 
 #ifdef HAVE_MYSQL
 	DBfield_definition_string(sql, sql_alloc, sql_offset, field);
@@ -321,20 +334,21 @@ static void	DBset_not_null_sql(char **sql, size_t *sql_alloc, size_t *sql_offset
 static void	DBadd_field_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
 		const char *table_name, const ZBX_FIELD *field)
 {
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table %s add ", table_name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table " ZBX_FS_SQL_NAME " add ", table_name);
 	DBfield_definition_string(sql, sql_alloc, sql_offset, field);
 }
 
 static void	DBrename_field_sql(char **sql, size_t *sql_alloc, size_t *sql_offset,
 		const char *table_name, const char *field_name, const ZBX_FIELD *field)
 {
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table %s ", table_name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "alter table " ZBX_FS_SQL_NAME " ", table_name);
 
 #ifdef HAVE_MYSQL
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "change column %s ", field_name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "change column " ZBX_FS_SQL_NAME " ", field_name);
 	DBfield_definition_string(sql, sql_alloc, sql_offset, field);
 #else
-	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "rename column %s to %s", field_name, field->name);
+	zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "rename column " ZBX_FS_SQL_NAME " to " ZBX_FS_SQL_NAME,
+			field_name, field->name);
 #endif
 }
 
@@ -389,8 +403,9 @@ static void	DBadd_foreign_key_sql(char **sql, size_t *sql_alloc, size_t *sql_off
 		const char *table_name, int id, const ZBX_FIELD *field)
 {
 	zbx_snprintf_alloc(sql, sql_alloc, sql_offset,
-			"alter table %s add constraint c_%s_%d foreign key (%s) references %s (%s)",
-			table_name, table_name, id, field->name, field->fk_table, field->fk_field);
+			"alter table " ZBX_FS_SQL_NAME " add constraint c_%s_%d foreign key (" ZBX_FS_SQL_NAME ")"
+					" references " ZBX_FS_SQL_NAME " (" ZBX_FS_SQL_NAME ")", table_name, table_name,
+					id, field->name, field->fk_table, field->fk_field);
 	if (0 != (field->fk_flags & ZBX_FK_CASCADE_DELETE))
 		zbx_strcpy_alloc(sql, sql_alloc, sql_offset, " on delete cascade");
 }
@@ -431,6 +446,22 @@ int	DBcreate_table(const ZBX_TABLE *table)
 	return ret;
 }
 
+int	DBrename_table(const char *table_name, const char *new_name)
+{
+	char	*sql = NULL;
+	size_t	sql_alloc = 0, sql_offset = 0;
+	int	ret = FAIL;
+
+	DBrename_table_sql(&sql, &sql_alloc, &sql_offset, table_name, new_name);
+
+	if (ZBX_DB_OK <= DBexecute("%s", sql))
+		ret = DBreorg_table(new_name);
+
+	zbx_free(sql);
+
+	return ret;
+}
+
 int	DBdrop_table(const char *table_name)
 {
 	char	*sql = NULL;
@@ -708,9 +739,8 @@ static int	DBcreate_dbversion_table(void)
 			ret = FAIL;
 		}
 	}
-	DBend(ret);
 
-	return ret;
+	return DBend(ret);
 }
 
 static int	DBset_version(int version, unsigned char mandatory)
@@ -741,6 +771,8 @@ extern zbx_dbpatch_t	DBPATCH_VERSION(3010)[];
 extern zbx_dbpatch_t	DBPATCH_VERSION(3020)[];
 extern zbx_dbpatch_t	DBPATCH_VERSION(3030)[];
 extern zbx_dbpatch_t	DBPATCH_VERSION(3040)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(3050)[];
+extern zbx_dbpatch_t	DBPATCH_VERSION(4000)[];
 
 static zbx_db_version_t dbversions[] = {
 	{DBPATCH_VERSION(2010), "2.2 development"},
@@ -753,6 +785,8 @@ static zbx_db_version_t dbversions[] = {
 	{DBPATCH_VERSION(3020), "3.2 maintenance"},
 	{DBPATCH_VERSION(3030), "3.4 development"},
 	{DBPATCH_VERSION(3040), "3.4 maintenance"},
+	{DBPATCH_VERSION(3050), "4.0 development"},
+	{DBPATCH_VERSION(4000), "4.0 maintenance"},
 	{NULL}
 };
 
@@ -886,13 +920,23 @@ int	DBcheck_version(void)
 
 	for (dbversion = dbversions; NULL != dbversion->patches; dbversion++)
 	{
-		zbx_dbpatch_t	*patches = dbversion->patches;
+		patches = dbversion->patches;
 
 		for (i = 0; 0 != patches[i].version; i++)
 		{
+			static sigset_t	orig_mask, mask;
+
 			if (db_optional >= patches[i].version)
 				continue;
 
+			/* block SIGTERM, SIGINT to prevent interruption of statements that cause an implicit commit */
+			sigemptyset(&mask);
+			sigaddset(&mask, SIGTERM);
+			sigaddset(&mask, SIGINT);
+
+			if (0 > sigprocmask(SIG_BLOCK, &mask, &orig_mask))
+				zabbix_log(LOG_LEVEL_WARNING, "cannot set sigprocmask to block the user signal");
+
 			DBbegin();
 
 			/* skipping the duplicated patches */
@@ -902,7 +946,10 @@ int	DBcheck_version(void)
 				ret = DBset_version(patches[i].version, patches[i].mandatory);
 			}
 
-			DBend(ret);
+			ret = DBend(ret);
+
+			if (0 > sigprocmask(SIG_SETMASK, &orig_mask, NULL))
+				zabbix_log(LOG_LEVEL_WARNING,"cannot restore sigprocmask");
 
 			if (SUCCEED != ret)
 				break;
