@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -41,12 +41,8 @@ const char	*zbx_json_strerror(void)
 	return zbx_json_strerror_message;
 }
 
-#ifdef HAVE___VA_ARGS__
-#	define zbx_set_json_strerror(fmt, ...) __zbx_zbx_set_json_strerror(ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
-#else
-#	define zbx_set_json_strerror __zbx_zbx_set_json_strerror
-#endif
-static void	__zbx_zbx_set_json_strerror(const char *fmt, ...)
+__zbx_attr_format_printf(1, 2)
+static void	zbx_set_json_strerror(const char *fmt, ...)
 {
 	va_list	args;
 
@@ -73,7 +69,7 @@ static void	__zbx_json_realloc(struct zbx_json *j, size_t need)
 		if (need > sizeof(j->buf_stat))
 		{
 			j->buffer_allocated = need;
-			j->buffer = zbx_malloc(j->buffer, j->buffer_allocated);
+			j->buffer = (char *)zbx_malloc(j->buffer, j->buffer_allocated);
 		}
 		else
 		{
@@ -97,11 +93,11 @@ static void	__zbx_json_realloc(struct zbx_json *j, size_t need)
 		if (j->buffer == j->buf_stat)
 		{
 			j->buffer = NULL;
-			j->buffer = zbx_malloc(j->buffer, j->buffer_allocated);
+			j->buffer = (char *)zbx_malloc(j->buffer, j->buffer_allocated);
 			memcpy(j->buffer, j->buf_stat, sizeof(j->buf_stat));
 		}
 		else
-			j->buffer = zbx_realloc(j->buffer, j->buffer_allocated);
+			j->buffer = (char *)zbx_realloc(j->buffer, j->buffer_allocated);
 	}
 }
 
@@ -170,6 +166,8 @@ static size_t	__zbx_json_stringsize(const char *string, zbx_json_type_t type)
 		{
 			case '"':  /* quotation mark */
 			case '\\': /* reverse solidus */
+			/* We do not escape '/' (solidus). https://www.rfc-editor.org/errata_search.php?rfc=4627 */
+			/* says: "/" and "\/" are both allowed and both produce the same result. */
 			case '\b': /* backspace */
 			case '\f': /* formfeed */
 			case '\n': /* newline */
@@ -178,7 +176,8 @@ static size_t	__zbx_json_stringsize(const char *string, zbx_json_type_t type)
 				len += 2;
 				break;
 			default:
-				if (0 != iscntrl(*sptr))
+				/* RFC 8259 requires escaping control characters U+0000 - U+001F */
+				if (0x1f >= (unsigned char)*sptr)
 					len += 6;
 				else
 					len++;
@@ -234,6 +233,8 @@ static char	*__zbx_json_insstring(char *p, const char *string, zbx_json_type_t t
 				*p++ = '\\';
 				*p++ = '\\';
 				break;
+			/* We do not escape '/' (solidus). https://www.rfc-editor.org/errata_search.php?rfc=4627 */
+			/* says: "/" and "\/" are both allowed and both produce the same result. */
 			case '\b':		/* backspace */
 				*p++ = '\\';
 				*p++ = 'b';
@@ -255,14 +256,15 @@ static char	*__zbx_json_insstring(char *p, const char *string, zbx_json_type_t t
 				*p++ = 't';
 				break;
 			default:
-				if (0 != iscntrl(*sptr))
+				/* RFC 8259 requires escaping control characters U+0000 - U+001F */
+				if (0x1f >= (unsigned char)*sptr)
 				{
 					*p++ = '\\';
 					*p++ = 'u';
 					*p++ = '0';
 					*p++ = '0';
-					*p++ = zbx_num2hex((*sptr >> 4) & 0xf);
-					*p++ = zbx_num2hex(*sptr & 0xf);
+					*p++ = zbx_num2hex((((unsigned char)*sptr) >> 4) & 0xf);
+					*p++ = zbx_num2hex(((unsigned char)*sptr) & 0xf);
 				}
 				else
 					*p++ = *sptr;
@@ -275,6 +277,21 @@ static char	*__zbx_json_insstring(char *p, const char *string, zbx_json_type_t t
 	return p;
 }
 
+void	zbx_json_escape(char **string)
+{
+	size_t	size;
+	char	*buffer;
+
+	if (0 == (size = __zbx_json_stringsize(*string, ZBX_JSON_TYPE_UNKNOWN)))
+		return;
+
+	buffer = zbx_malloc(NULL, size + 1);
+	buffer[size] = '\0';
+	__zbx_json_insstring(buffer, *string, ZBX_JSON_TYPE_UNKNOWN);
+	zbx_free(*string);
+	*string = buffer;
+}
+
 static void	__zbx_json_addobject(struct zbx_json *j, const char *name, int object)
 {
 	size_t	len = 2; /* brackets */
@@ -369,6 +386,50 @@ void	zbx_json_addstring(struct zbx_json *j, const char *name, const char *string
 	j->status = ZBX_JSON_COMMA;
 }
 
+void	zbx_json_addraw(struct zbx_json *j, const char *name, const char *data)
+{
+	size_t	len = 0, len_data;
+	char	*p, *psrc, *pdst;
+
+	assert(j);
+	len_data = strlen(data);
+
+	if (ZBX_JSON_COMMA == j->status)
+		len++; /* , */
+
+	if (NULL != name)
+	{
+		len += __zbx_json_stringsize(name, ZBX_JSON_TYPE_STRING);
+		len += 1; /* : */
+	}
+	len += len_data;
+
+	__zbx_json_realloc(j, j->buffer_size + len + 1/*'\0'*/);
+
+	psrc = j->buffer + j->buffer_offset;
+	pdst = j->buffer + j->buffer_offset + len;
+
+	memmove(pdst, psrc, j->buffer_size - j->buffer_offset + 1/*'\0'*/);
+
+	p = psrc;
+
+	if (ZBX_JSON_COMMA == j->status)
+		*p++ = ',';
+
+	if (NULL != name)
+	{
+		p = __zbx_json_insstring(p, name, ZBX_JSON_TYPE_STRING);
+		*p++ = ':';
+	}
+
+	memcpy(p, data, len_data);
+	p += len_data;
+
+	j->buffer_offset = p - j->buffer;
+	j->buffer_size += len;
+	j->status = ZBX_JSON_COMMA;
+}
+
 void	zbx_json_adduint64(struct zbx_json *j, const char *name, zbx_uint64_t value)
 {
 	char	buffer[MAX_ID_LEN];
@@ -385,6 +446,14 @@ void	zbx_json_addint64(struct zbx_json *j, const char *name, zbx_int64_t value)
 	zbx_json_addstring(j, name, buffer, ZBX_JSON_TYPE_INT);
 }
 
+void	zbx_json_addfloat(struct zbx_json *j, const char *name, double value)
+{
+	char	buffer[MAX_ID_LEN];
+
+	zbx_snprintf(buffer, sizeof(buffer), ZBX_FS_DBL, value);
+	zbx_json_addstring(j, name, buffer, ZBX_JSON_TYPE_INT);
+}
+
 int	zbx_json_close(struct zbx_json *j)
 {
 	if (1 == j->level)
@@ -869,7 +938,7 @@ static const char	*zbx_json_copy_value(const char *p, size_t len, char *out, siz
 	return p + len;
 }
 
-static const char	*zbx_json_decodevalue(const char *p, char *string, size_t size, int *is_null)
+const char	*zbx_json_decodevalue(const char *p, char *string, size_t size, int *is_null)
 {
 	size_t	len;
 
@@ -911,7 +980,7 @@ static const char	*zbx_json_decodevalue_dyn(const char *p, char **string, size_t
 			if (*string_alloc <= len)
 			{
 				*string_alloc = len + 1;
-				*string = zbx_realloc(*string, *string_alloc);
+				*string = (char *)zbx_realloc(*string, *string_alloc);
 			}
 
 			return zbx_json_copy_value(p, len, *string, *string_alloc);
@@ -922,7 +991,7 @@ static const char	*zbx_json_decodevalue_dyn(const char *p, char **string, size_t
 			if (*string_alloc < 1)
 			{
 				*string_alloc = 1;
-				*string = zbx_realloc(*string, *string_alloc);
+				*string = (char *)zbx_realloc(*string, *string_alloc);
 			}
 
 			**string = '\0';
@@ -1182,7 +1251,7 @@ static int	zbx_jsonpath_error(const char *path)
  *               FAIL    - json path parsing error                            *
  *                                                                            *
  ******************************************************************************/
-static int	zbx_jsonpath_next(const char *path, const char **pnext, zbx_strloc_t *loc, int *type)
+int	zbx_jsonpath_next(const char *path, const char **pnext, zbx_strloc_t *loc, int *type)
 {
 	const char	*next = *pnext;
 	size_t		pos;
@@ -1221,56 +1290,45 @@ static int	zbx_jsonpath_next(const char *path, const char **pnext, zbx_strloc_t
 	if ('[' != *next)
 		return zbx_jsonpath_error(*pnext);
 
-	while (*(++next) == ' ')
-		;
+	SKIP_WHITESPACE_NEXT(next);
 
 	/* process array index component */
 	if (0 != isdigit(*next))
 	{
-		for (pos = 0; 0 != isdigit(next[pos]); pos++)
+		for (pos = 1; 0 != isdigit(next[pos]); pos++)
 			;
 
-		if (0 == pos)
-			return zbx_jsonpath_error(*pnext);
-
 		loc->l = next - path;
 		loc->r = loc->l + pos - 1;
 
 		next += pos;
-
-		while (*next == ' ')
-			next++;
-
-		if (']' != *next++)
-			return zbx_jsonpath_error(*pnext);
-
-		*pnext = next;
 		*type = ZBX_JSONPATH_ARRAY_INDEX;
 
-		return SUCCEED;
+		SKIP_WHITESPACE(next);
 	}
-
-	loc->l = next - path + 1;
-
-	for (quotes = *next++; quotes != *next; next++)
+	else
 	{
-		if ('\0' == *next)
+		loc->l = next - path + 1;
+
+		for (quotes = *next++; quotes != *next; next++)
+		{
+			if ('\0' == *next)
+				return zbx_jsonpath_error(*pnext);
+		}
+
+		if ((pos = next - path) == loc->l)
 			return zbx_jsonpath_error(*pnext);
+
+		loc->r = pos - 1;
+		*type = ZBX_JSONPATH_COMPONENT_BRACKET;
+
+		SKIP_WHITESPACE_NEXT(next);
 	}
 
-	if ((pos = next - path) == loc->l)
-		return zbx_jsonpath_error(*pnext);
-
-	loc->r = pos - 1;
-
-	while (*(++next) == ' ')
-		;
-
 	if (']' != *next++)
 		return zbx_jsonpath_error(*pnext);
 
 	*pnext = next;
-	*type = ZBX_JSONPATH_COMPONENT_BRACKET;
 
 	return SUCCEED;
 }
@@ -1361,10 +1419,41 @@ void	zbx_json_value_dyn(const struct zbx_json_parse *jp, char **string, size_t *
 		size_t	len = jp->end - jp->start + 2;
 
 		if (*string_alloc < len)
-			*string = zbx_realloc(*string, len);
+			*string = (char *)zbx_realloc(*string, len);
 
 		zbx_strlcpy(*string, jp->start, len);
 	}
 }
 
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_json_path_check                                              *
+ *                                                                            *
+ * Purpose: validate json path string                                         *
+ *                                                                            *
+ * Parameters: path   - [IN] the json path                                    *
+ *             error  - [OUT] the error message buffer                        *
+ *             errlen - [IN] the size of error message buffer                 *
+ *                                                                            *
+ * Return value: SUCCEED - the json path component was parsed successfully    *
+ *               FAIL    - json path parsing error                            *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_json_path_check(const char *path, char * error, size_t errlen)
+{
+	const char	*next = NULL;
+	zbx_strloc_t	loc;
+	int		type;
 
+	do
+	{
+		if (SUCCEED != zbx_jsonpath_next(path, &next, &loc, &type))
+		{
+			zbx_snprintf(error, errlen, "json path not valid: %s", zbx_json_strerror());
+			return FAIL;
+		}
+	}
+	while ('\0' != *next);
+
+	return SUCCEED;
+}
