@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -212,32 +212,35 @@ static int	get_value(const char *source_ip, const char *host, unsigned short por
 	zbx_socket_t	s;
 	int		ret;
 	ssize_t		bytes_received = -1;
-	char		*tls_arg1, *tls_arg2, *request;
+	char		*tls_arg1, *tls_arg2;
 
-	if (ZBX_TCP_SEC_UNENCRYPTED == configured_tls_connect_mode)
+	switch (configured_tls_connect_mode)
 	{
-		tls_arg1 = NULL;
-		tls_arg2 = NULL;
-	}
-	else if (ZBX_TCP_SEC_TLS_CERT == configured_tls_connect_mode)
-	{
-		tls_arg1 = CONFIG_TLS_SERVER_CERT_ISSUER;
-		tls_arg2 = CONFIG_TLS_SERVER_CERT_SUBJECT;
-	}
-	else	/* ZBX_TCP_SEC_TLS_PSK */
-	{
-		tls_arg1 = CONFIG_TLS_PSK_IDENTITY;
-		tls_arg2 = NULL;		/* in case of TLS with PSK zbx_tls_connect() will find PSK */
+		case ZBX_TCP_SEC_UNENCRYPTED:
+			tls_arg1 = NULL;
+			tls_arg2 = NULL;
+			break;
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+		case ZBX_TCP_SEC_TLS_CERT:
+			tls_arg1 = CONFIG_TLS_SERVER_CERT_ISSUER;
+			tls_arg2 = CONFIG_TLS_SERVER_CERT_SUBJECT;
+			break;
+		case ZBX_TCP_SEC_TLS_PSK:
+			tls_arg1 = CONFIG_TLS_PSK_IDENTITY;
+			tls_arg2 = NULL;	/* zbx_tls_connect() will find PSK */
+			break;
+#endif
+		default:
+			THIS_SHOULD_NEVER_HAPPEN;
+			return FAIL;
 	}
 
 	if (SUCCEED == (ret = zbx_tcp_connect(&s, source_ip, host, port, GET_SENDER_TIMEOUT,
 			configured_tls_connect_mode, tls_arg1, tls_arg2)))
 	{
-		request = zbx_dsprintf(NULL, "%s\n", key);
-
-		if (SUCCEED == (ret = zbx_tcp_send(&s, request)))
+		if (SUCCEED == (ret = zbx_tcp_send(&s, key)))
 		{
-			if (0 < (bytes_received = zbx_tcp_recv_ext(&s, ZBX_TCP_READ_UNTIL_CLOSE, 0)))
+			if (0 < (bytes_received = zbx_tcp_recv_ext(&s, 0)))
 			{
 				if (0 == strcmp(s.buffer, ZBX_NOTSUPPORTED) && sizeof(ZBX_NOTSUPPORTED) < s.read_bytes)
 				{
@@ -258,8 +261,6 @@ static int	get_value(const char *source_ip, const char *host, unsigned short por
 			}
 		}
 
-		zbx_free(request);
-
 		zbx_tcp_close(&s);
 
 		if (SUCCEED != ret && 0 != bytes_received)
@@ -292,8 +293,11 @@ int	main(int argc, char **argv)
 	int		i, ret = SUCCEED;
 	char		*host = NULL, *key = NULL, *source_ip = NULL, ch;
 	unsigned short	opt_count[256] = {0}, port = ZBX_DEFAULT_AGENT_PORT;
+#if defined(_WINDOWS)
+	char		*error = NULL;
+#endif
 
-#ifndef _WINDOWS
+#if !defined(_WINDOWS) && (defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL))
 	if (SUCCEED != zbx_coredump_disable())
 	{
 		zbx_error("cannot disable core dump, exiting...");
@@ -382,6 +386,15 @@ int	main(int argc, char **argv)
 		}
 	}
 
+#if defined(_WINDOWS)
+	if (SUCCEED != zbx_socket_start(&error))
+	{
+		zbx_error(error);
+		zbx_free(error);
+		exit(EXIT_FAILURE);
+	}
+#endif
+
 	if (NULL == host || NULL == key)
 	{
 		usage();
@@ -431,10 +444,14 @@ int	main(int argc, char **argv)
 	{
 #if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
 		zbx_tls_validate_config();
+
+		if (ZBX_TCP_SEC_UNENCRYPTED != configured_tls_connect_mode)
+		{
 #if defined(_WINDOWS)
-		zbx_tls_init_parent();
+			zbx_tls_init_parent();
 #endif
-		zbx_tls_init_child();
+			zbx_tls_init_child();
+		}
 #endif
 	}
 #if !defined(_WINDOWS)
@@ -458,5 +475,10 @@ out:
 #endif
 	}
 #endif
+#if defined(_WINDOWS)
+	while (0 == WSACleanup())
+		;
+#endif
+
 	return SUCCEED == ret ? EXIT_SUCCESS : EXIT_FAILURE;
 }
