@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -174,13 +174,16 @@ int	zbx_add_event(unsigned char source, unsigned char object, zbx_uint64_t objec
 				substitute_simple_macros(NULL, &events[events_num], NULL, NULL, NULL, NULL, NULL, NULL,
 						NULL, &tag->value, MACRO_TYPE_TRIGGER_TAG, NULL, 0);
 
+				zbx_ltrim(tag->tag, ZBX_WHITESPACE);
+				zbx_ltrim(tag->value, ZBX_WHITESPACE);
+
 				if (TAG_NAME_LEN < zbx_strlen_utf8(tag->tag))
 					tag->tag[zbx_strlen_utf8_nchars(tag->tag, TAG_NAME_LEN)] = '\0';
 				if (TAG_VALUE_LEN < zbx_strlen_utf8(tag->value))
 					tag->value[zbx_strlen_utf8_nchars(tag->value, TAG_VALUE_LEN)] = '\0';
 
-				zbx_lrtrim(tag->tag, ZBX_WHITESPACE);
-				zbx_lrtrim(tag->value, ZBX_WHITESPACE);
+				zbx_rtrim(tag->tag, ZBX_WHITESPACE);
+				zbx_rtrim(tag->value, ZBX_WHITESPACE);
 
 				if (SUCCEED == validate_event_tag(&events[events_num], tag))
 					zbx_vector_ptr_append(&events[events_num].tags, tag);
@@ -704,8 +707,8 @@ static int	correlation_match_new_event(zbx_correlation_t *correlation, const DB_
 		else
 			value = "0";
 
-		zbx_replace_string(&expression, token.token.l, &token.token.r, value);
-		pos = token.token.r;
+		zbx_replace_string(&expression, token.loc.l, &token.loc.r, value);
+		pos = token.loc.r;
 	}
 
 	if (SUCCEED == evaluate(&result, expression, error, sizeof(error), NULL))
@@ -889,7 +892,7 @@ static char	*correlation_condition_get_event_filter(zbx_corr_condition_t *condit
 			{
 				tag = (zbx_tag_t *)event->tags.values[i];
 				if (0 == strcmp(tag->tag, condition->data.tag_pair.newtag))
-					zbx_vector_str_append(&values, DBdyn_escape_string(tag->value));
+					zbx_vector_str_append(&values, zbx_strdup(NULL, tag->value));
 			}
 
 			if (0 == values.values_num)
@@ -980,8 +983,8 @@ static int	correlation_add_event_filter(char **sql, size_t *sql_alloc, size_t *s
 			goto out;
 		}
 
-		zbx_replace_string(&expression, token.token.l, &token.token.r, filter);
-		pos = token.token.r;
+		zbx_replace_string(&expression, token.loc.l, &token.loc.r, filter);
+		pos = token.loc.r;
 		zbx_free(filter);
 	}
 
@@ -2436,7 +2439,7 @@ static void	process_trigger_events(zbx_vector_ptr_t *trigger_events, zbx_vector_
  ******************************************************************************/
 int	zbx_process_events(zbx_vector_ptr_t *trigger_diff, zbx_vector_uint64_t *triggerids_lock)
 {
-	const char		*__function_name = "process_events";
+	const char		*__function_name = "zbx_process_events";
 	size_t			i, processed_num = 0;
 	zbx_uint64_t		eventid;
 	zbx_vector_ptr_t	internal_ok_events, trigger_events;
