@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -39,11 +39,11 @@ ZBX_THREAD_ENTRY(selfmon_thread, args)
 
 	for (;;)
 	{
-		zbx_handle_log();
+		sec = zbx_time();
+		zbx_update_env(sec);
 
 		zbx_setproctitle("%s [processing data]", get_process_type_string(process_type));
 
-		sec = zbx_time();
 		collect_selfmon_stats();
 		sec = zbx_time() - sec;
 
@@ -51,9 +51,5 @@ ZBX_THREAD_ENTRY(selfmon_thread, args)
 				get_process_type_string(process_type), sec);
 
 		zbx_sleep_loop(ZBX_SELFMON_DELAY);
-
-#if !defined(_WINDOWS) && defined(HAVE_RESOLV_H)
-		zbx_update_resolver_conf();	/* handle /etc/resolv.conf update */
-#endif
 	}
 }
