@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 
 #include "daemon.h"
 #include "../../libs/zbxcrypto/tls.h"
+#include "../../libs/zbxserver/zabbix_stats.h"
 
 #define ZBX_MAX_SECTION_ENTRIES		4
 #define ZBX_MAX_ENTRY_ATTRIBUTES	3
@@ -167,12 +168,13 @@ static void	recv_proxyhistory(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx
 
 	if (SUCCEED != zbx_proxy_check_permissions(&proxy, sock, &error))
 	{
-		zabbix_log(LOG_LEVEL_WARNING, "cannot accept connection from proxy \"%s\" at \"%s\": %s",
-				proxy.host, sock->peer, error);
+		zabbix_log(LOG_LEVEL_WARNING, "cannot accept connection from proxy \"%s\" at \"%s\", allowed address:"
+				" \"%s\": %s", proxy.host, sock->peer, proxy.proxy_address, error);
 		goto out;
 	}
 
-	zbx_proxy_update_version(&proxy, jp);
+	zbx_update_proxy_data(&proxy, zbx_get_protocol_version(jp), time(NULL),
+			(0 != (sock->protocol & ZBX_TCP_COMPRESS) ? 1 : 0));
 
 	if (SUCCEED != (ret = process_proxy_history_data(&proxy, jp, ts, &error)))
 	{
@@ -180,10 +182,10 @@ static void	recv_proxyhistory(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx
 				proxy.host, sock->peer, error);
 		goto out;
 	}
-
-	update_proxy_lastaccess(proxy.hostid, time(NULL));
 out:
-	zbx_send_response(sock, ret, error, CONFIG_TIMEOUT);
+	/* 'history data' request is sent only by pre 3.4 version proxies */
+	/* that did not have compression support                          */
+	zbx_send_response_ext(sock, ret, error, NULL, ZBX_TCP_PROTOCOL, CONFIG_TIMEOUT);
 
 	zbx_free(error);
 
@@ -229,7 +231,7 @@ static void	recv_proxy_heartbeat(zbx_socket_t *sock, struct zbx_json_parse *jp)
 	const char	*__function_name = "recv_proxy_heartbeat";
 
 	char		*error = NULL;
-	int		ret;
+	int		ret, flags = ZBX_TCP_PROTOCOL;
 	DC_PROXY	proxy;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
@@ -243,16 +245,21 @@ static void	recv_proxy_heartbeat(zbx_socket_t *sock, struct zbx_json_parse *jp)
 
 	if (SUCCEED != (ret = zbx_proxy_check_permissions(&proxy, sock, &error)))
 	{
-		zabbix_log(LOG_LEVEL_WARNING, "cannot accept connection from proxy \"%s\" at \"%s\": %s",
-				proxy.host, sock->peer, error);
+		zabbix_log(LOG_LEVEL_WARNING, "cannot accept connection from proxy \"%s\" at \"%s\", allowed address:"
+				" \"%s\": %s", proxy.host, sock->peer, proxy.proxy_address, error);
 		goto out;
 	}
 
-	zbx_proxy_update_version(&proxy, jp);
+	zbx_update_proxy_data(&proxy, zbx_get_protocol_version(jp), time(NULL),
+			(0 != (sock->protocol & ZBX_TCP_COMPRESS) ? 1 : 0));
 
-	update_proxy_lastaccess(proxy.hostid, time(NULL));
+	if (0 != proxy.auto_compress)
+		flags |= ZBX_TCP_COMPRESS;
 out:
-	zbx_send_response(sock, ret, error, CONFIG_TIMEOUT);
+	if (FAIL == ret && 0 != (sock->protocol & ZBX_TCP_COMPRESS))
+		flags |= ZBX_TCP_COMPRESS;
+
+	zbx_send_response_ext(sock, ret, error, NULL, flags, CONFIG_TIMEOUT);
 
 	zbx_free(error);
 
@@ -323,7 +330,7 @@ static void	queue_stats_export(zbx_hashset_t *queue_stats, const char *id_name,
 
 	zbx_hashset_iter_reset(queue_stats, &iter);
 
-	while (NULL != (stats = zbx_hashset_iter_next(&iter)))
+	while (NULL != (stats = (zbx_queue_stats_t *)zbx_hashset_iter_next(&iter)))
 	{
 		zbx_json_addobject(json, NULL);
 		zbx_json_adduint64(json, id_name, stats->id);
@@ -340,7 +347,7 @@ static void	queue_stats_export(zbx_hashset_t *queue_stats, const char *id_name,
 }
 
 /* queue item comparison function used to sort queue by nextcheck */
-static int	queue_compare_by_nextcheck_asc(void **d1, void **d2)
+static int	queue_compare_by_nextcheck_asc(zbx_queue_item_t **d1, zbx_queue_item_t **d2)
 {
 	zbx_queue_item_t	*i1 = *d1, *i2 = *d2;
 
@@ -376,7 +383,7 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_SID, sessionid, sizeof(sessionid)) ||
 			SUCCEED != DBget_user_by_active_session(sessionid, &user) || USER_TYPE_SUPER_ADMIN > user.type)
 	{
-		zbx_send_response_raw(sock, ret, "Permission denied.", CONFIG_TIMEOUT);
+		zbx_send_response(sock, ret, "Permission denied.", CONFIG_TIMEOUT);
 		goto out;
 	}
 
@@ -397,7 +404,7 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 			if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_LIMIT, limit_str, sizeof(limit_str)) ||
 					FAIL == is_uint31(limit_str, &limit))
 			{
-				zbx_send_response_raw(sock, ret, "Unsupported limit value.", CONFIG_TIMEOUT);
+				zbx_send_response(sock, ret, "Unsupported limit value.", CONFIG_TIMEOUT);
 				goto out;
 			}
 		}
@@ -405,7 +412,7 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 
 	if (ZBX_GET_QUEUE_UNKNOWN == request_type)
 	{
-		zbx_send_response_raw(sock, ret, "Unsupported request type.", CONFIG_TIMEOUT);
+		zbx_send_response(sock, ret, "Unsupported request type.", CONFIG_TIMEOUT);
 		goto out;
 	}
 
@@ -424,14 +431,14 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 			/* gather queue stats by item type */
 			for (i = 0; i < queue.values_num; i++)
 			{
-				zbx_queue_item_t	*item = queue.values[i];
+				zbx_queue_item_t	*item = (zbx_queue_item_t *)queue.values[i];
 				zbx_uint64_t		id = item->type;
 
-				if (NULL == (stats = zbx_hashset_search(&queue_stats, &id)))
+				if (NULL == (stats = (zbx_queue_stats_t *)zbx_hashset_search(&queue_stats, &id)))
 				{
 					zbx_queue_stats_t	data = {.id = id};
 
-					stats = zbx_hashset_insert(&queue_stats, &data, sizeof(data));
+					stats = (zbx_queue_stats_t *)zbx_hashset_insert(&queue_stats, &data, sizeof(data));
 				}
 				queue_stats_update(stats, now - item->nextcheck);
 			}
@@ -449,14 +456,14 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 			/* gather queue stats by proxy hostid */
 			for (i = 0; i < queue.values_num; i++)
 			{
-				zbx_queue_item_t	*item = queue.values[i];
+				zbx_queue_item_t	*item = (zbx_queue_item_t *)queue.values[i];
 				zbx_uint64_t		id = item->proxy_hostid;
 
-				if (NULL == (stats = zbx_hashset_search(&queue_stats, &id)))
+				if (NULL == (stats = (zbx_queue_stats_t *)zbx_hashset_search(&queue_stats, &id)))
 				{
 					zbx_queue_stats_t	data = {.id = id};
 
-					stats = zbx_hashset_insert(&queue_stats, &data, sizeof(data));
+					stats = (zbx_queue_stats_t *)zbx_hashset_insert(&queue_stats, &data, sizeof(data));
 				}
 				queue_stats_update(stats, now - item->nextcheck);
 			}
@@ -475,7 +482,7 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 
 			for (i = 0; i < queue.values_num && i < limit; i++)
 			{
-				zbx_queue_item_t	*item = queue.values[i];
+				zbx_queue_item_t	*item = (zbx_queue_item_t *)queue.values[i];
 
 				zbx_json_addobject(&json, NULL);
 				zbx_json_adduint64(&json, "itemid", item->itemid);
@@ -491,7 +498,7 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "%s() json.buffer:'%s'", __function_name, json.buffer);
 
-	(void)zbx_tcp_send_raw(sock, json.buffer);
+	(void)zbx_tcp_send(sock, json.buffer);
 
 	DCfree_item_queue(&queue);
 	zbx_vector_ptr_destroy(&queue);
@@ -570,10 +577,10 @@ out:
 
 /* auxiliary variables for status_stats_export() */
 
-zbx_entry_info_t	templates, hosts_monitored, hosts_not_monitored, items_active_normal, items_active_notsupported,
+static zbx_entry_info_t	templates, hosts_monitored, hosts_not_monitored, items_active_normal, items_active_notsupported,
 			items_disabled, triggers_enabled_ok, triggers_enabled_problem, triggers_disabled, users_online,
 			users_offline, required_performance;
-int			templates_res, users_res;
+static int		templates_res, users_res;
 
 static void	zbx_status_counters_init(void)
 {
@@ -801,7 +808,7 @@ static void	status_stats_export(struct zbx_json *json, zbx_user_type_t access_le
 					{
 						const zbx_proxy_counter_t	*proxy_counter;
 
-						proxy_counter = entry->info->counters.values[i];
+						proxy_counter = (zbx_proxy_counter_t *)entry->info->counters.values[i];
 						status_entry_export(json, entry, proxy_counter->counter_value,
 								&proxy_counter->proxyid);
 					}
@@ -847,7 +854,7 @@ static int	recv_getstatus(zbx_socket_t *sock, struct zbx_json_parse *jp)
 	if (SUCCEED != zbx_json_value_by_name(jp, ZBX_PROTO_TAG_SID, sessionid, sizeof(sessionid)) ||
 			SUCCEED != DBget_user_by_active_session(sessionid, &user))
 	{
-		zbx_send_response_raw(sock, ret, "Permission denied.", CONFIG_TIMEOUT);
+		zbx_send_response(sock, ret, "Permission denied.", CONFIG_TIMEOUT);
 		goto out;
 	}
 
@@ -865,7 +872,7 @@ static int	recv_getstatus(zbx_socket_t *sock, struct zbx_json_parse *jp)
 
 	if (ZBX_GET_STATUS_UNKNOWN == request_type)
 	{
-		zbx_send_response_raw(sock, ret, "Unsupported request type.", CONFIG_TIMEOUT);
+		zbx_send_response(sock, ret, "Unsupported request type.", CONFIG_TIMEOUT);
 		goto out;
 	}
 
@@ -890,7 +897,7 @@ static int	recv_getstatus(zbx_socket_t *sock, struct zbx_json_parse *jp)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "%s() json.buffer:'%s'", __function_name, json.buffer);
 
-	(void)zbx_tcp_send_raw(sock, json.buffer);
+	(void)zbx_tcp_send(sock, json.buffer);
 
 	zbx_json_free(&json);
 
@@ -905,15 +912,107 @@ out:
 #undef ZBX_GET_STATUS_FULL
 }
 
+/******************************************************************************
+ *                                                                            *
+ * Function: send_internal_stats_json                                         *
+ *                                                                            *
+ * Purpose: process Zabbix stats request                                      *
+ *                                                                            *
+ * Parameters: sock  - [IN] the request socket                                *
+ *             jp    - [IN] the request data                                  *
+ *                                                                            *
+ * Return value:  SUCCEED - processed successfully                            *
+ *                FAIL - an error occurred                                    *
+ *                                                                            *
+ ******************************************************************************/
+static int	send_internal_stats_json(zbx_socket_t *sock, const struct zbx_json_parse *jp)
+{
+	const char	*__function_name = "send_internal_stats_json";
+	struct zbx_json	json;
+	char		type[MAX_STRING_LEN], error[MAX_STRING_LEN];
+	int		ret = FAIL;
+
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+
+	if (NULL == CONFIG_STATS_ALLOWED_IP ||
+			SUCCEED != zbx_tcp_check_allowed_peers(sock, CONFIG_STATS_ALLOWED_IP))
+	{
+		zabbix_log(LOG_LEVEL_WARNING, "failed to accept an incoming stats request: %s",
+				NULL == CONFIG_STATS_ALLOWED_IP ? "StatsAllowedIP not set" : zbx_socket_strerror());
+		strscpy(error, "Permission denied.");
+		goto out;
+	}
+
+	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
+
+	if (SUCCEED == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_TYPE, type, sizeof(type)) &&
+			0 == strcmp(type, ZBX_PROTO_VALUE_ZABBIX_STATS_QUEUE))
+	{
+		char			from_str[ZBX_MAX_UINT64_LEN + 1], to_str[ZBX_MAX_UINT64_LEN + 1];
+		int			from = ZBX_QUEUE_FROM_DEFAULT, to = ZBX_QUEUE_TO_INFINITY;
+		struct zbx_json_parse	jp_data;
+
+		if (SUCCEED != zbx_json_brackets_by_name(jp, ZBX_PROTO_TAG_PARAMS, &jp_data))
+		{
+			zbx_snprintf(error, sizeof(error), "cannot find tag: %s", ZBX_PROTO_TAG_PARAMS);
+			goto param_error;
+		}
+
+		if (SUCCEED == zbx_json_value_by_name(&jp_data, ZBX_PROTO_TAG_FROM, from_str, sizeof(from_str))
+				&& FAIL == is_time_suffix(from_str, &from, ZBX_LENGTH_UNLIMITED))
+		{
+			strscpy(error, "invalid 'from' parameter");
+			goto param_error;
+		}
+
+		if (SUCCEED == zbx_json_value_by_name(&jp_data, ZBX_PROTO_TAG_TO, to_str, sizeof(to_str)) &&
+				FAIL == is_time_suffix(to_str, &to, ZBX_LENGTH_UNLIMITED))
+		{
+			strscpy(error, "invalid 'to' parameter");
+			goto param_error;
+		}
+
+		if (ZBX_QUEUE_TO_INFINITY != to && from > to)
+		{
+			strscpy(error, "parameters represent an invalid interval");
+			goto param_error;
+		}
+
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, ZBX_PROTO_VALUE_SUCCESS, ZBX_JSON_TYPE_STRING);
+		zbx_json_adduint64(&json, ZBX_PROTO_VALUE_ZABBIX_STATS_QUEUE, DCget_item_queue(NULL, from, to));
+	}
+	else
+	{
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, ZBX_PROTO_VALUE_SUCCESS, ZBX_JSON_TYPE_STRING);
+		zbx_json_addobject(&json, ZBX_PROTO_TAG_DATA);
+
+		zbx_get_zabbix_stats(&json);
+
+		zbx_json_close(&json);
+	}
+
+	(void)zbx_tcp_send(sock, json.buffer);
+	ret = SUCCEED;
+param_error:
+	zbx_json_free(&json);
+out:
+	if (SUCCEED != ret)
+		zbx_send_response(sock, ret, error, CONFIG_TIMEOUT);
+
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+
+	return ret;
+}
+
 static void	active_passive_misconfig(zbx_socket_t *sock)
 {
-	char   *msg = NULL;
+	char	*msg = NULL;
 
 	msg = zbx_dsprintf(msg, "misconfiguration error: the proxy is running in the active mode but server at \"%s\""
 			" sends requests to it as to proxy in passive mode", sock->peer);
 
 	zabbix_log(LOG_LEVEL_WARNING, "%s", msg);
-	zbx_send_response(sock, FAIL, msg, CONFIG_TIMEOUT);
+	zbx_send_proxy_response(sock, FAIL, msg, CONFIG_TIMEOUT);
 	zbx_free(msg);
 }
 
@@ -1035,6 +1134,10 @@ static int	process_trap(zbx_socket_t *sock, char *s, zbx_timespec_t *ts)
 				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
 					ret = recv_getstatus(sock, &jp);
 			}
+			else if (0 == strcmp(value, ZBX_PROTO_VALUE_ZABBIX_STATS))
+			{
+				ret = send_internal_stats_json(sock, &jp);
+			}
 			else
 				zabbix_log(LOG_LEVEL_WARNING, "unknown request received [%s]", value);
 		}
@@ -1153,8 +1256,7 @@ ZBX_THREAD_ENTRY(trapper_thread, args)
 		/* Only after receiving data it is known who has sent them and one can decide to accept or discard */
 		/* the data. */
 		ret = zbx_tcp_accept(&s, ZBX_TCP_SEC_TLS_CERT | ZBX_TCP_SEC_TLS_PSK | ZBX_TCP_SEC_UNENCRYPTED);
-		sec = zbx_time();
-		zbx_update_env(sec);
+		zbx_update_env(zbx_time());
 
 		if (SUCCEED == ret)
 		{
@@ -1168,6 +1270,7 @@ ZBX_THREAD_ENTRY(trapper_thread, args)
 			zbx_setproctitle("%s #%d [processing data]", get_process_type_string(process_type),
 					process_num);
 
+			sec = zbx_time();
 			process_trapper_child(&s, &ts);
 			sec = zbx_time() - sec;
 
